/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * https://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

/**
 * Classe principale de l'application mathGraph32 permettant de créer ou modifier une figure
 * @constructor
 * @param {SVGElement} svg Le svg dans lequel l'application travaille (il doit avoir un id)
 * @param {MtgOptions} mtgOptions Les informations sur l'initialisation de l'application
 */
const MtgApp = require('./MtgAppBase')
const CListeObjets = require('./objets/CListeObjets')
const NatObj = require('./types/NatObjAdd')
const NatCal = require('./types/NatCal')
const Nat = require('./types/Nat')
const kernel = require('./kernel/kernel')
const DataInputStream = require('./entreesSorties/DataInputStream')
const DataOutputStream = require('./entreesSorties/DataOutputStream')
const CMathGraphDoc = require('./objets/CMathGraphDoc')
const CValeur = require('./objets/CValeur')
const CCalcul = require('./objets/CCalcul')
const CConstante = require('./objets/CConstante')
const CImplementationProto = require('./objets/CImplementationProto')
const CImage = require('./objets/CImage')
const CSegment = require('./objets/CSegment')
const CPolygone = require('./objets/CPolygone')
const CCommentaire = require('./objets/CCommentaire')
const CNoeudPointeurSurPoint = require('./objets/CNoeudPointeurSurPoint')
const CAffLiePt = require('./objets/CAffLiePt')
const StyleEncadrement = require('./types/StyleEncadrement')
const Dimf = require('./types/Dimf')
const constantes = require('./kernel/constantes')
const kernelAdd = require('./kernel/kernelAdd')
const ButtonTool = require('./interface/ButtonTool')
const Color = require('./types/Color')
const $ = require('jquery')
const AvertDlg = require('./dialogs/AvertDlg')
const ConfirmDlg = require('./dialogs/ConfirmDlg')
const CValeurAngle = require('./objets/CValeurAngle')

require('../css/mtgApp.css')

// on exporte MtgAppBase en ajoutant plein de méthodes à son prototype
module.exports = MtgApp

/**
 * Fonction initialisant la variable tipDisplayed de tous les boutons à false;
 */
MtgApp.prototype.annuleTipsButtons = function () {
  for (var i = 0; i < this.buttons.length; i++) this.buttons[i].tipDisplayed = false
}

MtgApp.prototype.updateActiveTools = function () {
  for (var i = 0; i < this.expandableBars.length; i++) {
    this.expandableBars[i].updateActiveTools()
  }
}

MtgApp.prototype.updateActiveIcons = function () {
  var row = 0
  for (var i = 0; i < this.expandableBars.length; i++) {
    if (this.expandableBars[i].tools.length !== 0) this.expandableBars[i].updateActiveIcon(row++)
    else this.expandableBars[i].desactive()
  }
}

MtgApp.prototype.updateToolsToolBar = function () {
  // Il faut d'abord cacher toutes les icônes de tous les outils
  for (var i = 0; i < this.expandableBars.length; i++) this.expandableBars[i].hideAllIcons()
  // Puis on remet à jour les outils actifs et leurs icônes
  this.updateActiveTools()
  this.updateActiveIcons()
}

MtgApp.prototype.unFoldExpandableBars = function () {
  var row = 0
  for (var i = 0; i < this.expandableBars.length; i++) {
    var b = this.expandableBars[i]
    if (b.tools.length !== 0) {
      if (b.expanded) b.updateActiveIcon(row)
      row++
    }
  }
  // On remontre les éditeurs de formule qui ont été masqués lorsque la barre a été dépliée
  this.showInputs(true)
}

MtgApp.prototype.hasBarExpanded = function () {
  var res = false
  for (var i = 0; i < this.expandableBars.length; i++) {
    res = res || this.expandableBars[i].expanded
    if (res) break
  }
  return res
}

MtgApp.prototype.getExpandedBar = function () {
  for (var i = 0; i < this.expandableBars.length; i++) {
    if (this.expandableBars[i].expanded) return this.expandableBars[i]
  }
  return null
}

/**
 * Fonction rajoutant à la figure actuelle des graduations en utilisant la construction nommée nomProto
 * @param nomProto
 */
MtgApp.prototype.creeGrad = function (nomProto) {
  var proto, nbgradx, nbgrady, listeSources
  var list = this.doc.listePr
  var repere = list.premierParNatCal(NatCal.NRepere)
  nbgradx = new CCalcul(list, null, false, 'nbgradx', '20', new CConstante(list, 20))
  list.add(nbgradx)
  nbgrady = new CCalcul(list, null, false, 'nbgrady', '20', new CConstante(list, 20))
  list.add(nbgrady)
  proto = this.docCons.getPrototype(nomProto)
  listeSources = new CListeObjets()
  listeSources.add(nbgradx)
  listeSources.add(nbgrady)
  proto.get(0).elementAssocie = repere // Pointe sur le repère
  proto.get(1).elementAssocie = nbgradx
  proto.get(2).elementAssocie = nbgrady
  var impProto = new CImplementationProto(list, proto)
  impProto.implemente(this.dimf, proto)
}

/**
 * Fonction préparant la liste principale pour que la figure possède un segment longueur unité
 * @param uniteAngle L'unité d'angle de la figure
 */
MtgApp.prototype.initAvecLongueurUnite = function (uniteAngle) {
  var list = this.doc.listePr
  list.uniteAngle = uniteAngle
  list.ajouteConstantePi()
  list.ajouteLongueurUnite(this.dimf)
}
/**
 * Fonction préparant la liste principale pour que la figure ne possède pas un segment longueur unité
 * @param uniteAngle L'unité d'angle de la figure
 */
MtgApp.prototype.initSansLongueurUnite = function (uniteAngle) {
  var list = this.doc.listePr
  list.uniteAngle = uniteAngle
  list.ajouteConstantePi()
  list.pointeurLongueurUnite = null
}

/**
 * Fonction préparant la liste principale pour que la figure possède un repère orthonormal avec graduations
 * @param uniteAngle L'unité d'angle de la figure
 * @param quadhor true si on veut que le repère soit quadrillé horizontalement
 * @param quadver true si on veut que le repère soit quadrillé verticalement
 * @param grid true si on veut que le repère est des pointilés aux points de coordonnées entières
 * @param withvect si true, on rajoute des vecteurs sur les axes
 * @param typegrad String valant "no" pour pas de graduations, "trig" pour une graduation spéciale trigo, "simple"
 */
MtgApp.prototype.initAvecRepereOrthonormal = function (uniteAngle, quadhor, quadver, grid, withvect, typegrad) {
  var list = this.doc.listePr
  list.uniteAngle = uniteAngle
  list.ajouteConstantePi()
  list.ajouteRepereOrthonormal(this.dimf, quadhor, quadver, grid, 'O', 'I', 'J', withvect ? 'i' : '',
    withvect ? 'j' : '', new CValeur(list, 0), new CValeur(list, 0), new CValeur(list, 1), new CValeur(list, 1),
    typegrad !== 'no')
  if (typegrad !== 'no') { this.creeGrad(typegrad === 'trig' ? 'GraduationReperePourTrigo' : 'GraduationAxesRepere') }
  if (typegrad === 'trig') list.uniteAngle = kernel.uniteAngle.radian
}

/**
 * Fonction préparant la liste principale pour que la figure possède un repère orthogonal avec graduations
 * @param uniteAngle L'unité d'angle de la figure
 * @param quadhor true si on veut que le repère soit quadrillé horizontalement
 * @param quadver true si on veut que le repère soit quadrillé verticalement
 * @param grid true si on veut que le repère est des pointilés aux points de coordonnées entières
 * @param withvect si true, on rajoute des vecteurs sur les axes
 * @param typegrad String valant "no" pour pas de graduations, "trig" pour une graduation spéciale trigo, "simple"
 */
MtgApp.prototype.initAvecRepereOrthogonal = function (uniteAngle, quadhor, quadver, grid, withvect, typegrad) {
  var list = this.doc.listePr
  list.uniteAngle = uniteAngle
  list.ajouteConstantePi()
  list.ajouteRepereOrthonormal(this.dimf, quadhor, quadver, grid, 'O', 'I', 'J', withvect ? 'i' : '',
    withvect ? 'j' : '', new CValeur(list, 0), new CValeur(list, 0), new CValeur(list, 1), new CValeur(list, 1),
    typegrad !== 'no')
  if (typegrad !== 'no') { this.creeGrad(typegrad === 'trig' ? 'GraduationReperePourTrigo' : 'GraduationAxesRepere') }
  if (typegrad === 'trig') list.uniteAngle = kernel.uniteAngle.radian
}
/// /////////////////////////////////////////////////////////////////////////////////////////////////////////////
//          Ce qui suit concerne les exercices de construction en ligne
/// /////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 * Fonction renvoyant true si la figure chargée est une figure destinée à faire un exercice de construction.
 * Pour cela la figure doit contenir une macro d'apparition d'objets d'intitulé #Solution#.
 * Les objets que cette macro fait apparaître son alors considérés comme les objets que l'élève doit construire.
 */
MtgApp.prototype.isExercise = function () {
  return this.macroPourConst !== null
}

/**
 * Fonction cherchant s'il existe une macro d'apparition d'objets d'intitulé #Solution# ou #SolutionIso# et renvoyant
 * un pointeur sur celle-ci si elle existe et sinon null.
 * Dans le cas #SolutionIso#, la macro d'apparition doit avoir un seul objet qui soit un polygone
 * @returns {*}
 */
MtgApp.prototype.getMacroPourConst = function () {
  var list = this.listePr
  for (var i = 0; i < list.longueur(); i++) {
    var el = list.get(i)
    if (el.className === 'CMacroApparition') {
      if (['#Solution#', '#SolutionIso#'].indexOf(el.intitule) !== -1) {
        // Dans les deux cas d'un exercice de construction isométrique, si un des objets à construire est un polygone, tous ses sommets doivent être nommés
        if (el.intitule === '#Solution#') return el
        else {
          if (el.intitule === '#SolutionIso#') {
            // Pour être valide, la macro de doit gérer que des objets de type Polygone, cercle ou segment
            var valide = true
            var listeAssociee = el.listeAssociee
            for (var j = 0; (j < listeAssociee.longueur()) && valide; j++) {
              var el2 = listeAssociee.get(j)
              // Si demande de construire un polygone, tous ses sommets doivent être nommés dans la figure initiale
              if (el2.estDeNature(NatObj.NPolygone)) {
                var nbp = el2.nombrePoints
                for (var k = 0; (k < nbp) && valide; k++) {
                  valide = valide && el2.colPoints[k].pointeurSurPoint.nom !== ''
                }
              } else {
                // Si on demande de créer un segment, les extrémités doivent être nommées
                if (el2.estDeNature(NatObj.NSegment)) {
                  valide = valide && (el2.point1.nom !== '') && (el2.point2.nom !== '')
                } else {
                  // Si demande de créer un cercle, il faut qu'il soit de type CCercleOA ou CCercleOR ou CCercleOAB
                  if (el2.estCercleParCentre()) {
                    valide = valide && (el2.o.nom !== '')
                  } else valide = false // Pas d'autres types d'objets à construire que polygône, segment ou cercle
                }
              }
              valide = valide &&
                listeAssociee.get(j).estDeNature(Nat.or(NatObj.NPolygone, NatObj.NCercle, NatObj.NSegment))
            }
            if (valide) return el
          }
        }
      }
    }
  }
  return null
}

/**
 * Fonction appelée pour le cas d'un exercice de construction et renvoyant une liste formée de tous les
 * objets (numériques ou non) que l'élève a le droit d'utiliser pour résoudre l'exercice.
 * Pour un exercice de construction, il contient une macro d'apparition d'objets.
 * Pour tous les calculs nommés (saut les constantes comme pi) si le commentaire de cette macro contient
 * une chaine de caractères du type {nomducalcul} alors le calcul ou la fonction nommé nomducalcul peut
 * être utilisée par l'élève
 * @returns {CListeObjets}
 */
MtgApp.prototype.listePourConstruction = function () {
  if (!this.estExercice) return this.listePr
  var list = this.listePr
  var nbObj = this.nbObjInit // Le nombre initial d'objets de la figure de l'exercice
  // var li = new CListeObjets(list.uniteAngle, list.pointeurLongueurUnite)
  var li = new CListeObjets(list.uniteAngle) // Corrigé version 6.5.2
  li.associeA(this.doc)
  for (var i = 0; i < list.longueur(); i++) {
    var el = list.get(i)
    if (!el.estElementIntermediaire()) {
      if (i < nbObj) {
        if (el.estDeNature(NatObj.NTtObj) && !el.masque) li.add(el)
        else {
          if (el.estDeNatureCalcul(NatCal.NCalculReelConstant)) li.add(el) // Pour la constante pi
          else if (el.estDeNatureCalcul(NatCal.NTtCalcNommeSaufConst)) {
            if (this.calculOKForConst(el)) li.add(el)
          }
        }
      } else li.add(el)
    }
  }
  return li
}

/**
 * Fonction appelée dans le cas d'un exercice de construction/
 * Renvoie true si calc fait partie des calcul que l'élève a le droit d'utiliser
 * @param calc
 * @returns {boolean}
 */
MtgApp.prototype.calculOKForConst = function (calc) {
  var mac = this.macroPourConst
  return mac.commentaireMacro.indexOf('{' + calc.nomCalcul + '}') !== -1
}

MtgApp.prototype.arrayObjAConstruire = function () {
  var mac = this.macroPourConst
  var ar = []
  var list = mac.listeAssociee
  for (var i = 0; i < list.longueur(); i++) {
    var el = list.get(i)
    ar.push(el.index) // el.index contient l'indice de el dans la liste qui le contient
  }
  return ar
}

/**
 * Fonction déplaçant un point libre ou un point lié un tout petit peude façon aléatoire
 * @param pt Le point libre ou lié à déplacer
 */
MtgApp.prototype.epsilon = function (pt) {
  var a, b, s1, s2, x1, y1, absmin, absmax, nabs
  if (pt.estDeNature(NatObj.NPointBase)) {
    a = (Math.floor(Math.random() * 5) + 1) * 0.001
    b = (Math.floor(Math.random() * 5) + 1) * 0.001
    s1 = Math.floor(Math.random() * 2)
    s2 = Math.floor(Math.random() * 2)
    x1 = s1 === 0 ? a : -a
    y1 = s2 === 0 ? b : -b
    pt.placeEn(pt.x + x1, pt.y + y1)
  } else { // Cas d'un point lié
    absmin = pt.abscisseMinimale()
    absmax = pt.abscisseMaximale()
    // on décale l'abscisse au maximum de 5% vers la haut ou vers la bas au
    // hasard
    a = Math.floor(Math.random() * 5) + 1
    s1 = Math.floor(Math.random() * 2)
    if (s1 === 0) a = -a
    nabs = pt.abscisse + a * (absmax - absmin) / 10000
    if (nabs > absmax) nabs = absmax
    else if (nabs < absmin) nabs = absmin
    pt.donneAbscisse(nabs)
  }
}

MtgApp.prototype.validateAnswerOnePass = function (list) {
  for (var i = 0; i < this.arrayObjAConst.length; i++) {
    var el = list.get(this.arrayObjAConst[i])
    var name = el.estDeNature(NatObj.NObjNommable) ? el.nom : ''
    var conf = false
    for (var j = this.nbObjInit; (j < list.longueur()) && !conf; j++) {
      var e = list.get(j)
      conf = conf || (e.coincideAvec(el) && ((name.length !== 0) ? (e.nom === el.nom) : true))
    }
    if (!conf) return false
  }
  return true
}

MtgApp.prototype.validateAnswerIso = function () {
  var list = this.listePr
  for (var i = 0; i < this.arrayObjAConst.length; i++) {
    var el = list.get(this.arrayObjAConst[i])
    var conf = false
    for (var j = this.nbObjInit; (j < list.longueur()) && !conf; j++) {
      conf = conf || (el.isIsomTo(list.get(j)))
    }
    // Pour le cas particulier d'un polygone on regarde si l'équivalent a été créé avec des segments
    if (!conf && el.estDeNature(NatObj.NPolygone) && this.existePolyEq(el)) {
      // On va créer un polygone provisoire dont les sommets sont les points construits ayant les mêmes noms que les
      // sommets de el
      var col = new Array(el.nombrePoints)
      for (j = 0; j < el.nombrePoints; j++) {
        var pt1 = el.colPoints[j].pointeurSurPoint
        var pt2 = this.pointParNom(pt1.nom)
        col[j] = new CNoeudPointeurSurPoint(list, pt2)
      }
      var poly = new CPolygone(list, null, false, el.couleur, false, el.style, col)
      poly.positionne()
      conf = conf || el.isIsomTo(poly)
    }
    if (!conf) return false
  }
  return true
}

/**
 * Ne sert que pour les exercices de construction
 * Fonction renvoyant true s'il a été construit un segment, une droite ou une demi droite auquel appatiennent
 * les points construits nommés nom1 et nom2
 * @param {string} nom1
 * @param {string} nom2
 */
MtgApp.prototype.existeObjDte = function (nom1, nom2) {
  var list = this.listePr
  var pt1 = this.pointParNom(nom1)
  var pt2 = this.pointParNom(nom2)
  if (pt1 === null || pt2 === null) return false
  // for (var i = this.nbObjInit; i < list.longueur(); i++) { // On accepte aussi les objets lignes présents dans la figure initiale.
  for (var i = 0; i < list.longueur(); i++) {
    var el = list.get(i)
    if (el.estDeNature(NatObj.NTteDroite) && ((i <= this.nbObjInit) ? !el.masque : true)) {
      if (pt1.appartientParDefinition(el) && pt2.appartientParDefinition(el)) return true
    }
  }
  return false
}

/**
 * Fonction utilisée seulement pour les exercices de construction.
 * Renvoie le point ayant pour nom nom sauf si ce point fait partie de la figure initiale et est masque.
 * Renvoie null s'il n'y a pas de tel point.
 * Normalement ne doit pas renvoyer null car n'est appelé que si tous les points ont été nommés comme demandé.
 * @param nom
 * @returns {*}
 */
MtgApp.prototype.pointParNom = function (nom) {
  var list = this.listePr
  // for (var i = this.nbObjInit; i < list.longueur(); i++) {
  for (var i = 0; i < list.longueur(); i++) {
    var el = list.get(i)
    if (el.estDeNature(NatObj.NTtPoint) && (el.nom === nom) && ((i <= this.nbObj) ? !el.masque : true)) return el
  }
  return null
}

/**
 * Ne sert que pour les exercices de construction
 * Fonction renvoyant true s'il a été constuit de segments joignant tous les sommets du polygone poly
 * @param {CPolygone} poly
 */
MtgApp.prototype.existePolyEq = function (poly) {
  var existe = true
  for (var i = 0; i < poly.nombrePoints - 1; i++) {
    var nom1 = poly.colPoints[i].pointeurSurPoint.nom
    var nom2 = poly.colPoints[i + 1].pointeurSurPoint.nom
    existe = existe && this.existeObjDte(nom1, nom2)
  }
  return existe
}

/*
MtgApp.prototype.getMissingNames = function() {
  var ret = [];
  var list = this.listePr;
  for (var i = 0; i < this.arrayObjAConst.length; i++) {
    var el = list.get(this.arrayObjAConst[i]);
    if (el.estDeNature(NatObj.NObjNommable)) {
      if (el.nom.length !== 0) {
        var valid = false;
        for (var j = this.nbObjInit; (j < list.longueur()) && !valid; j++) {
          var e = list.get(j);
          if (e.estDeNature(NatObj.NObjNommable)) valid = valid || (e.nom === el.nom);
        }
        if (!valid) ret.push(el.nom);
      }
    }
  }
  return ret;
}
*/

MtgApp.prototype.getMissingNames = function () {
  var i, j, k, el, ch, valid, noeud, pt, pt1, pt2
  var ret = []
  var list = this.listePr
  // On crée d'abord un tableau formé de tous les noms des points et droites qui doivent être nommés
  var tab = []
  for (i = 0; i < this.arrayObjAConst.length; i++) {
    el = list.get(this.arrayObjAConst[i])
    if (el.estDeNature(NatObj.NObjNommable) && (el.nom !== '')) tab.push(el.nom)
    else {
      if (el.estDeNature(NatObj.NPolygone)) {
        for (k = 0; k < el.nombrePoints - 1; k++) {
          noeud = el.colPoints[k]
          pt = noeud.pointeurSurPoint
          ch = pt.nom
          if (pt.masque && (ch !== '') && (tab.indexOf(ch) === -1)) tab.push(ch)
        }
      } else {
        if (el.className === 'CSegment') {
          pt1 = el.point1
          ch = pt1.nom
          if (pt1.masque && (ch !== '')) tab.push(ch)
          pt2 = el.point2
          ch = pt2.nom
          if (pt2.masque && ch !== '') tab.push(ch)
        } else {
          if (el.estCercleParCentre()) {
            pt = el.o
            ch = pt.nom
            if (pt.masque && (ch !== '')) tab.push(ch)
          }
        }
      }
    }
  }
  for (i = 0; i < tab.length; i++) {
    valid = false
    ch = tab[i]
    for (j = this.nbObjInit; (j < list.longueur()) && !valid; j++) {
      var e = list.get(j)
      if (e.estDeNature(NatObj.NObjNommable)) valid = valid || (e.nom === ch)
    }
    if (!valid) ret.push(ch)
  }

  return ret
}

/**
 * Fonction utilisée dans les exercices de constructions et renvoyant un tableau formé de chaînes de caractères
 * correspondat aux types graphiques d'éléments qui auraient dû être créée et ne l'ont pas été.
 * Renvoie une tableau vide s'il ne manque d'éléments.
 * @returns {Array}
 */
MtgApp.prototype.getMissingTypes = function () {
  var ret = []
  var list = this.listePr
  for (var i = 0; i < this.arrayObjAConst.length; i++) {
    var el = list.get(this.arrayObjAConst[i])
    // Pour le exercices de construction isométrique quand on demande un polygone on accepte l'équivalent tracé avec des segments
    if ((this.macroPourConst.intitule === '#Solution#') || (this.macroPourConst.intitule === '#SolutionIso#' && !el.estDeNature(NatObj.NPolygone))) {
      var valid = false
      for (var j = this.nbObjInit; (j < list.longueur()) && !valid; j++) {
        var e = list.get(j)
        valid = e.estDeNature(kernelAdd.natObjGraphPourProto(el.getNature()))
      }
      var ch = kernelAdd.chaineNatGraphPourProto(el.getNature())
      if (!valid && ret.indexOf(ch)) ret.push(ch)
    }
  }
  return ret
}

MtgApp.prototype.validateAnswer = function () {
  var i, j
  // On regarde d'abord s'il s'agit de construire un seul objet isométrique à un objet donné
  if (this.macroPourConst.intitule === '#SolutionIso#') return this.validateAnswerIso()
  else {
    // S'il n'existe pas des objets créés par l'utilisateur coincidant avec les objets à construire
    // la construction est fausse
    if (!this.validateAnswerOnePass(this.listePr)) return false
    // Sinon on crée une nouvelle liste formée des clones des objets de la liste principale
    // On déplace de epsilon trois fois de suites tous les points libres ou points liés et on regarde
    // si la réponse est encore valide.
    var list = this.listePr
    var listAux = new CListeObjets(list.uniteAngle)
    list.setCopy(listAux)
    var listAConstruire = new CListeObjets()
    for (i = 0; i < this.arrayObjAConst.length; i++) listAConstruire.add(list.get(this.arrayObjAConst[i]))
    var nat = Nat.or(NatObj.NPointBase, NatObj.NPointLie)
    for (j = 0; (j < 3); j++) {
      for (i = 0; i < this.nbObjInit; i++) {
        var el = listAux.get(i)
        if (el.estDeNature(nat) && listAConstruire.depDe(el)) this.epsilon(el)
      }
      listAux.positionne(false, this.dimf)
      if (!this.validateAnswerOnePass(listAux)) return false
    }
    return true
  }
}

/**
 * Fonctionrenvoyant, dans le cas d'un exercice, true si l'utilisateur a rajouté des objets
 * à la figure initiale.
 * @returns {boolean}
 */
MtgApp.prototype.objectConstructed = function () {
  return this.listePr.longueur() > this.nbObjInit
}

/**
 * Fonction remplaçant la figure actuelle par la figure dont le code Base64 est code
 * @param code Le code Base64 de la nouvelle figure
 * @param bdisplay Si true, la 6figure est affichée
 */
MtgApp.prototype.setFigByCode = function (code, bdisplay = true) {
  var doc
  try {
    // doc = new CMathGraphDoc(this.id + "figurePanel", true, true); // Modifié version 6.3.0
    doc = new CMathGraphDoc(this.id, true, true)
    var ba = kernel.base64Decode(code)
    var inps = new DataInputStream(ba)
    doc.read(inps)
  } catch (e) {
    new AvertDlg(this, e.message)
    return
  }
  this.doc = doc
  // Ligne suivante nécessaire pour que les macros d'activation et désactivation du mode trace activent ou non
  // l'icône correspondante
  this.doc.app = this
  var list = this.listePr
  list.deleteComponents()
  list.retireTout()
  this.retireTout()
  this.prepareTracesEtImageFond()
  doc.dimf = this.dimf
  this.listePr = doc.listePr
  this.listePourConst = this.listePourConstruction()
  this.updateToolsToolBar()
  this.creeCommentaireDesignation()
  this.listePr.creePaneVariables()
  this.calculate(false)
  if (bdisplay) this.display()
  if (this.cadre !== null) this.createCadre(this.widthCadre, this.heightCadre)
  // Ligne ci-dessous modifiée. On appelle this.gestionnaire.initialise(); car cette fonction peut-être appelée depuis j3p
  // et il faut dans ce cas qu'on ne puisse pas annuler ce qui a été fait avant.
  // if (this.electron) this.gestionnaire.initialise(); else this.gestionnaire.enregistreFigureEnCours("NewFig");
  this.gestionnaire.initialise()
  this.reInitConst() // Pour réinitialiser une éventuelle construction en cours
  // if (doc.modeTraceActive) this.buttonModeTrace.activate(true);
}

/**
 * Rend la figure éactive ou inative aux événements souris et clavier
 * suivant la valeur du boolean ba.
 * @param {boolean} ba
 * @returns {undefined}
 */
MtgApp.prototype.setActive = function (ba) {
  if (this.nameEditor.isVisible) this.nameEditor.montre(false)
  this.activeOutilCapt()
  var doc = this.doc
  if (doc !== null) {
    doc.isActive = ba
    // Il faut aussi activer ou désactiver les champs d'édition de formule
    var liste = doc.listePr
    for (var i = 0; i < liste.longueur(); i++) {
      var el = liste.get(i)
      if (el.estDeNature(NatObj.NEditeurFormule)) el.editeur.readOnly = !ba
      else {
        if (el.getNatureCalcul() === NatCal.NVariable) {
          if (el.dialogueAssocie) {
            el.buttonplus.setAttribute('disabled', !ba)
            el.buttonmoins.setAttribute('disabled', !ba)
            el.buttonegal.setAttribute('disabled', !ba)
          }
        }
      }
    }
  }
}

MtgApp.prototype.natPourImages = function () {
  var mac = this.macroPourConst
  var nat = NatObj.NTtPoint
  if (mac === null) return this.onlyPoints ? nat : NatObj.NTtObjPourTransf
  else {
    var com = mac.commentaireMacro
    var tab1 = ['droite', 'segment', 'demidroite', 'cercle', 'arc', 'polygone', 'lignebrisee']
    var tab2 = [NatObj.NDroite, NatObj.NSegment, NatObj.NDemiDroite, NatObj.NCercle, NatObj.NArc, NatObj.NPolygone, NatObj.NLigneBrisee]
    for (var i = 0; i < tab1.length; i++) {
      if (com.indexOf('#' + tab1[i]) !== -1) nat = Nat.or(nat, tab2[i])
    }
    return nat
  }
}

/**
 * Fonction changeant la formule du calcul ou de
 * la fonction (réelle ou complexe) de nom nomCalcul.
 * La nouvelle formule est contenue dans la chaîne de caractères formule.
 * Renvoie true si la formule était valide et false sinon.
 * Utilisé par j3p pour certains exercices de construction
 * @param {String} nomCalcul
 * @param {String} formule
 * @returns {undefined}
 */
MtgApp.prototype.giveFormula2 = function (nomCalcul, formule) {
  this.listePr.giveFormula2(nomCalcul, formule)
}

/**
 * Fonction renvoyant le code LaTeX de l'affichage LaTex d'indice ind dans la liste
 * des objets créés (les indices commençant à zéro)
 * Utilisé par j3p pour certains exercices de construction
 * @param {number} ind
 * @returns {String}
 */
/**
 * Renvoie la valeur actuelle du calcul réel nommé nomCalcul dans la figure.
 * Renvoie -1 si le calcul n'existe pas.
 * @param {String} nomCalcul
 * @returns {number}
 */
MtgApp.prototype.valueOf = function (nomCalcul) {
  return this.listePr.valueOf(nomCalcul)
}

MtgApp.prototype.getLatexCode = function (ind) {
  return this.listePr.getLatexCode(ind)
}

/**
 * Fonction recosntruisant une figure à partir d'un flux binaire
 * @param ba Tableau de bytes contenant le flux binaire de la figure
 * @param callBackOnOK Fonction éventelle de callBack à appeler après chargement de la figrue
 * @param {string} filePath  Le chemin éventuel d'accès à la figure si on ouvre depuis le disque
 */
MtgApp.prototype.resetFromByteArray = function (ba, callBackOnOK = null, filePath = '') {
  var self = this
  try {
    var inps = new DataInputStream(ba)
    // var doc = new CMathGraphDoc(self.id + "figurePanel", true, true); // Modifié version 6.3.0
    var doc = new CMathGraphDoc(self.id, true, true)
    doc.read(inps)
  } catch (e) {
    new AvertDlg(self, e.message)
    return
  }
  self.nameEditor.montre(false) // Des fois qu'un nom soit en cours d'édition

  var list = self.listePr
  list.deleteComponents()
  list.retireTout()
  self.retireTout()
  doc.dimf = self.dimf
  doc.app = self
  self.doc = doc
  // Ligne suivante nécessaire pour que les macros d'activation et désactivation du mode trace activent ou non
  // l'icône correspondante
  self.doc.app = self
  self.updateOpacitySlider()
  self.prepareTracesEtImageFond()
  self.listePr = doc.listePr
  self.listePourConst = self.listePr
  self.updateToolsToolBar()
  self.creeCommentaireDesignation()
  this.listePr.creePaneVariables()
  self.calculateAndDisplay(false)
  // if (doc.modeTraceActive) self.buttonModeTrace.activate(true);
  if (self.cadre !== null) self.createCadre(self.widthCadre, self.heightCadre)
  if (self.electron) self.gestionnaire.initialise(); else self.gestionnaire.enregistreFigureEnCours('NewFig')
  self.reInitConst() // Pour réinitialiser une éventuelle construction en cours
  if (this.electron) {
    doc.setDirty(true, false)
    // On appelle setNewPath qui est une fonction de index.html sert à autoriser main.js à changer le chemin d'accès de la figure
    setNewPath(filePath) // eslint-disable-line no-undef
  }
  this.updateToolbar()
  self.activeOutilDem()
  if (callBackOnOK !== null) callBackOnOK()
}

/**
 * Fonction chargeant une figure depuis une chaîne de caractères (en utf-8)
 * Cette fonction est utilisée par la version electron
 * @param ch La chaîne de caractères contenant le code de la figure (chaque caractère a le code Ascii
 * permettant de créer un ByteArray représentant le flux binaire de la figure.
 * @param callBackOnOK
 */
MtgApp.prototype.resetFromString = function (ch, callBackOnOK) {
  var ba = []
  for (var i = 0; i < ch.length; i++) ba.push(ch.charCodeAt(i))
  this.resetFromByteArray(ba, callBackOnOK)
}

/**
 * Fonction chargeant une figure depuis un objet File
 * @param file
 * @param callBackOnOK Fonction de callBack à appeler si le chargement a réussi
 */
MtgApp.prototype.resetFromFile = function (file, callBackOnOK) {
  var reader = new FileReader() // eslint-disable-line no-undef
  reader.readAsArrayBuffer(file)
  var self = this
  reader.onload = function () {
    try {
      var ba = new Uint8Array(reader.result)
      self.resetFromByteArray(ba, callBackOnOK, file.path)
    } catch (e) {
      new AvertDlg(self, e.message)
    }
  }
}

/**
 * Fonction utilisée par la version electron quand on double-clique sur un fichier pour lancer le logiciel
 * @param ch Contient une chaîne de caractères contenant un flux binaire représentant une figure
 * @returns {CMathGraphDoc} Renvoie null si le code n'est pas valide et sinon le CMathGraphDoc correspondant
 */
MtgApp.prototype.getDocFromString = function (ch) {
  try {
    var ba = []
    for (var i = 0; i < ch.length; i++) ba.push(ch.charCodeAt(i))
    var inps = new DataInputStream(ba)
    // var doc = new CMathGraphDoc(this.id + "figurePanel", true, true); // Modifié version 6.3.0
    var doc = new CMathGraphDoc(this.id, true, true)
    doc.read(inps)
    return doc
  } catch (e) {
    return null
  }
}

/**
 * Fonction ajoutant un prototype à la figure depuis un tavbeau d'entier ba
 * @param ba
 */
MtgApp.prototype.addProtoFromByteArray = function (ba) {
  var self = this
  try {
    var inps = new DataInputStream(ba)
    this.doc.addPrototype(inps, this.doc.numeroVersion)
  } catch (e) {
    new AvertDlg(self, 'FichierErr')
  }
}

MtgApp.prototype.addProtoFromString = function (ch) {
  var ba = []
  for (var i = 0; i < ch.length; i++) ba.push(ch.charCodeAt(i))
  this.addProtoFromByteArray(ba)
}

MtgApp.prototype.addProtoFromFile = function (file) {
  var reader = new FileReader() // eslint-disable-line no-undef
  reader.readAsArrayBuffer(file)
  var self = this
  reader.onload = function () {
    var ba = new Uint8Array(reader.result)
    self.addProtoFromByteArray(ba)
  }
}

MtgApp.prototype.onDropFile = function (ev) {
  if (ev.dataTransfer && ev.dataTransfer.files && ev.dataTransfer.files.length) {
    this.activeOutilCapt()
    ev.stopPropagation()
    kernel.preventDefault(ev)
    const file = ev.dataTransfer.files[0]
    let imgType = file.name.split('.')
    imgType = imgType[imgType.length - 1].toLowerCase() // On utilise toLowerCase() pour éviter les extensions en majuscules
    const allowedTypes = ['mgj', 'png', 'jpg', 'jpeg', 'gif']
    const ind = allowedTypes.indexOf(imgType)
    if (ind !== -1) {
      if (ind === 0) { // Cas d'un fichier mtg32
        if (this.doc.isDirty) {
          var self = this
          new ConfirmDlg(this, 'AvertDirty', function () {
            self.resetFromFile(file, function () {
            })
            // Si on est dans la version electron on remet à zéro le document
            // resetDocument est une fonction de la page index.html de la vesion electron
            if (self.electron) resetDocument() // eslint-disable-line no-undef
          })
        } else {
          this.resetFromFile(file, function () {
          })
        }
      } else { // cas d'un fichier image
        const self = this
        const natImage = DataOutputStream.getNatImage(imgType)
        const reader = new FileReader() // eslint-disable-line no-undef
        reader.readAsDataURL(file)
        reader.addEventListener('load', function () {
          const database64 = reader.result
          const tab = database64.split(',')
          const point = kernelAdd.mousePosition(self.svgFigure, ev, self.zoomFactor) // 3 ième paramètre rajouté version 6.5.2
          // reader.result contient la chaîne Base64 représentant l'image
          const im = new CImage(self.listePr, null, false, self.getCouleur(), point.x, point.y, 0, 0, false, null, 13,
            StyleEncadrement.Sans, false, self.doc.couleurFond, CAffLiePt.alignHorCent,
            CAffLiePt.alignVerLow, natImage, kernel.base64Decode(tab[tab.length - 1], true))
          const image = kernel.ce('img', {
            src: database64
          })
          image.onload = function () {
            im.width = image.width
            im.height = image.height
            self.ajouteElement(im)
            im.creeAffichage(self.svgFigure, false, self.doc.couleurFond, self.doc.opacity)
            self.outilActif.saveFig()
            self.activeOutilCapt()
          }
        }, false)
      }
    }
  }
}

MtgApp.prototype.createCadre = function (width, height) {
  this.widthCadre = width
  this.heightCadre = height
  this.cadre = kernel.cens('rect', {
    x: 2,
    y: 2,
    width: width,
    height: height,
    rx: 4,
    ry: 4,
    style: 'fill:none;stroke:lightgrey;stroke-width:4;'
  })
  this.svgFigure.insertBefore(this.cadre, this.doc.gTraces)
}

MtgApp.prototype.deleteCadre = function () {
  this.svgFigure.removeChild(this.cadre)
  this.cadre = null
}

/**
 * Fonction créant la représentation de la figure dans un URI et une fois que c'est prêt appelant
 * la fonction de callBack() callBack avec comme paramètre le blob représentant l'image
 * @param {string} imageType  "png" ou "jpeg"
 * @param svg Le svg dans lequel se fait l'affichage
 * @param callBack La fonction de callBack
 * @param fileName Le nom du fichier pour l'enregistrement
 * @param {number} coefMult Le coefficient multiplicateur utilisé pour gagner (ou perdre ) en définition
 * @param {number}coef Coefficient d'agrandissement-réduction utilisé pour ,'exportation en pNG avec unité
 */
/* Abandonné
MtgApp.prototype.getImageBlobData = function(imageType, svg, fileName, callBack, coefMult, coef) {
  var dimf = this.dimf;
  var avecCadre = this.cadre !== null;
  // var coefMult = this.pref_coefMult; // Le coefficient multiplicateur pour la taille des images
  var w = (avecCadre ? this.widthCadre : dimf.x)*coefMult;
  var h = (avecCadre ? this.heightCadre : dimf.y)*coefMult;

  var canvas = kernel.ce("canvas", {
    width : w * coef, // On utilise round pour arrondir à la valeur entière la plus proche
    height : h * coef
    // style : "width:" + w + ";height:" + h
  });

  var ctx = canvas.getContext('2d');
  var data = (new XMLSerializer()).serializeToString(svg);
  var DOMURL = window.URL || window.webkitURL || window;
  var img = new Image();
  var svgBlob = new Blob([data], {type: 'image/svg+xml;charset=utf-8'});
  var url = DOMURL.createObjectURL(svgBlob);

  img.onload = function () {
    ctx.drawImage(img, 0, 0);
    DOMURL.revokeObjectURL(url);
    // callBack(canvas.toDataURL('image/png').replace('image/png', 'image/octet-stream'));
    // callBack(fileName, canvas.toDataURL('image/png'));
    canvas.toBlob(
      function(blob) {
        callBack(fileName, blob);
      },
      "image/" + imageType
    )
  };

  img.src = url;
}
*/
/**
 * Fonction appelée par la version electron pour obtenir le codeBase64 correspondant à la figure en PNG
 * @param {string} imageType  "png" ou "jpeg"
 * @param {float} coef le coefficient d'agrandissement ou rédution utilisé pour l'exportation en PNG avec unité
 * @param {number} coefMult Le coefficient multiplicateur utilisé pour gagner (ou perdre ) en définition
 * @param callBack
 */
MtgApp.prototype.getBase64ImageData = function (imageType, callBack, coefMult, coef) {
  var doc = this.doc
  var listePr = this.listePr
  // Si un cadre a été choisi dans la figure et si la case à cocher est cochée, on enregistre la figure
  // avec les dimensions du cadre
  var avecCadre = this.cadre !== null
  // var coefMult = this.pref_coefMult; // Le coefficient multiplicateur pour la taille des images
  var w = (avecCadre ? this.widthCadre : doc.dimf.x) * coefMult
  var h = (avecCadre ? this.heightCadre : doc.dimf.y) * coefMult
  var dimf = new Dimf(w, h)
  var svg = kernel.cens('svg', {
    width: w * coef,
    height: h * coef,
    viewBox: '0 0 ' + w + ' ' + h
  })
  /// /////////////////////////////////////////////////////////////////////////////////////////////////
  // Très important : Il faut cloner les defs de MathJax contenues dans le document pour que les LaTeX
  // puissent être tracés correctement
  /* Plus nécessaire version 6.4 avec MathJax 3
  var defs = document.getElementById("MathJax_SVG_glyphs");
  svg.appendChild(defs.cloneNode(true));
  */
  // Et il faut aussi cloner les pattern pour les remplissages de surface
  const defs = document.getElementById('mtg32_patterns')
  const defsclone = svg.appendChild(defs.cloneNode(true))
  // On remplace les patterns de remplissage par des deux fois plus gros pour exportation
  var coefarr = (coefMult < 1) ? 1 : Math.round(coefMult) // On arrondit à l'entier le plus proche si >= 1
  var dimRectPattern = String(coefarr * 6)
  for (var i = 0; i < listePr.longueur(); i++) {
    var el = listePr.get(i)
    if (el.estDeNature(NatObj.NSurface) && (el.pattern !== null)) {
      // Modification version 6.3.2 pour rendre le player compatible avec les hachures
      // var pattern = $(defsclone).children("#" + el.index + "mtg32pattern")[0];
      var pattern = $(defsclone).children('#' + listePr.id + el.index + 'mtg32pattern')[0]
      $(pattern).attr('width', dimRectPattern).attr('height', dimRectPattern)
      pattern.childNodes[0].setAttribute('d', el.getPattern(true, coefarr))
    }
  }
  /// /////////////////////////////////////////////////////////////////////////////////////////////////
  // Il faut cloner la liste principale car sinon on a des problèmes de rafraichissement des g elements
  // des objets graphiques
  var list = new CListeObjets(listePr)
  list.isForExport = true // Sert à supprimer l'éditeur de l'export.
  this.listePr.setCopy(list)
  list.coefMult = coefMult // Utilisé pour la trace des points et des marques d'angles
  list.adaptRes(coefMult)
  list.createComponents(svg)
  list.positionne(false, dimf)
  list.setReady4MathJax(false, false)
  MathJax.Hub.Queue(function () {
    svg.appendChild(kernel.cens('rect', {
      width: '100%',
      height: '100%',
      fill: doc.couleurFond.rgb()
    }))
    if (doc.imageFond !== null) {
      var g = kernel.cens('image', {
        x: '0',
        y: '0',
        width: doc.widthImageFond * coefMult,
        height: doc.heightImageFond * coefMult
      })
      // g.setAttributeNS("http://www.w3.org/1999/xlink", "href", "data:image/png;base64," + kernel.base64Encode(doc.imageFond, true));
      g.setAttributeNS('http://www.w3.org/1999/xlink', 'href', 'data:image/' + imageType + ';base64,' + kernel.base64Encode(doc.imageFond, true))
      svg.appendChild(g)
    }
    var traces = doc.gTraces.cloneNode(true)
    $(traces).attr('transform', 'scale(' + coefMult + ')')
    svg.appendChild(traces)
    list.afficheTout(0, svg, true, doc.couleurFond, doc.opacity, true) // Dernier paramètre ajouté version 4.6.1
    list.deleteComponents()
    // Version 4.6 : Pour que les patterns de remplissages restent les bons il faut les supprimer et les recréer
    $('#mtg32_patterns').empty()
    for (i = 0; i < listePr.longueur(); i++) {
      el = listePr.get(i)
      if (el.estDeNature(NatObj.NSurface) && (el.pattern !== null)) el.createPattern()
    }

    var canvas = kernel.ce('canvas', {
      width: w * coef, // On utilise round pour arrondir à la valeur entière la plus proche
      height: h * coef
      // style : "width:" + w + ";height:" + h
    })

    var ctx = canvas.getContext('2d')
    // On supprime les pointer-events de la réprésentation de la figure
    var data = (new XMLSerializer()).serializeToString(svg).replace(/pointer-events[ ]*=[ ]*(['"])none\1/g, '') // eslint-disable-line no-undef
    // Si on veut l'exportation en svg tout est déjà dans data
    if (imageType === 'svg') callBack(data)
    else {
      var DOMURL = window.URL || window.webkitURL || window
      var img = new Image() // eslint-disable-line no-undef
      var svgBlob = new Blob([data], { type: 'image/svg+xml;charset=utf-8' }) // eslint-disable-line no-undef
      var url = DOMURL.createObjectURL(svgBlob)
      img.onload = function () {
        ctx.drawImage(img, 0, 0)
        DOMURL.revokeObjectURL(url)
        callBack(canvas.toDataURL('image/' + imageType).replace(/^data:image\/\w+;base64,/, ''))
      }
      img.src = url
    }
  })
}

// Les fonctions suivantes sont des fonctions uniquement utiles pour la version electron

/**
 * Fonction utilisée par electron et renvoyant un tableau d'entiers contenant le code binaire de la figure
 * @returns {Array|*}
 */
MtgApp.prototype.getByteArrayCode = function () {
  var oups = new DataOutputStream()
  // Si un cadre a été choisi dans la figure et si la case à cocher est cochée, on enregistre la figure
  // avec les dimensions du cadre
  var doc = this.doc
  var olddimf = this.dimf
  var avecCadre = (this.cadre !== null)
  if (avecCadre) doc.dimf = new Dimf(this.widthCadre, this.heightCadre)
  doc.write(oups)
  doc.dimf = olddimf
  return oups.ba
}

/**
 * Fonction utilisée pour la version elctron et renvoyan un tableau d'entiers contenant le code binaire
 * du protoytype n° ind de la figure
 * @param ind
 */
MtgApp.prototype.getProtoByteArrayCode = function (ind) {
  var oups = new DataOutputStream()
  this.doc.tablePrototypes[ind].write(oups)
  return oups.ba
}

MtgApp.prototype.resize = function (w, h) {
  var doc = this.doc
  var svg = this.svg
  var svgFig = this.svgFigure
  var zf = this.zoomFactor
  var wp = w + 'px'
  var hp = h + 'px'
  var div = svg.parentNode // Le div parent
  $(div).css('width', wp).css('height', hp)

  $(svg).attr('width', w).attr('height', h)
  $(this.rightPanel).attr('x', String(w - constantes.rightPanelWidth * zf)).attr('y', String(constantes.toolbarHeight * zf + 26))
  $('#mtgleftpanel').attr('width', w).attr('height', h)
  this.dimf = new Dimf(w - constantes.svgPanelWidth * zf - constantes.rightPanelWidth * zf, h - constantes.topIconSize * zf)
  $('#svgrect').attr('width', this.dimf.x).attr('height', this.dimf.y)
  $('#divsvgaux').css('width', wp).css('height', hp)
  $('#divMtgDlg').css('width', wp).css('height', hp)

  $(svgFig).attr('width', this.dimf.x).attr('height', this.dimf.y)
  doc.dimf = this.dimf
  this.nameEditor.montre(false) // Des fois qu'un nom soit en cours d'édition
  var list = this.listePr
  list.positionne(false, this.dimf)
  list.update(svgFig, doc.couleurFond, doc.opacity, true)
  // On met à jour les div éventuels associés à des variables.
  var ind = 0
  for (var i = list.longueur() - 1; i >= 0; i--) {
    var el = list.get(i)
    if (el.getNatureCalcul() === NatCal.NVariable) {
      if (el.dialogueAssocie) {
        var x = w - el.div.offsetWidth
        var y = h - el.div.offsetHeight * (ind + 1)
        $(el.div).css('left', x + 'px')
        $(el.div).css('top', y + 'px')
        ind++
      }
    }
  }
}

MtgApp.prototype.addToolsSup = function () {
  // A revoir une fois les constructions au point
  var toolsNameArray = ['CreationConst', 'GestionConst', 'CopierStyle', 'TaillePlus', 'TailleMoins', 'ReclassDebObjGra', 'ReclassFinObjGra']
  // var toolsNameArray = ["CopierStyle", "TaillePlus", "TailleMoins", "ReclassDebObjGra", "ReclassFinObjGra"];
  var svg = this.svg
  var zoomFactor = this.zoomFactor
  var nbIcones = toolsNameArray.length
  var width = parseFloat(svg.getAttribute('width'))
  var iconSize = constantes.topIconSize
  var w = nbIcones * (iconSize + 2) * zoomFactor
  var h = constantes.topIconSize * zoomFactor
  this.svgToolsAdd = kernel.cens('svg', {
    id: 'svgToolsAdd',
    width: w,
    height: h,
    x: width - w,
    y: iconSize * zoomFactor + 26
  })
  svg.appendChild(this.svgToolsAdd)
  this.svgToolsAdd.style.cursor = 'default'
  for (var row = 0; row < nbIcones; row++) {
    var toolName = toolsNameArray[row]
    var button = new ButtonTool(this, toolName, 'float', false, row)
    this['button' + toolName] = button
    this.svgToolsAdd.appendChild(button.container)
  }
}

/**
 * Fonction ajoutant à listePr un segment d'extrémités pt1 et pt2 dans les styles actifs
 * à condition qu'un tel objet n'ait pas déjà été défini
 * @param pt1
 * @param pt2
 */
MtgApp.prototype.addSegment = function (pt1, pt2) {
  var li = this.listePr
  var seg = new CSegment(li, null, false, this.getCouleur(), false, this.getStyleTrait(), pt1, pt2)
  seg.positionne(false, this.dimf)
  if (!this.existeDeja(seg)) li.add(seg)
}

MtgApp.prototype.addPolygon = function () {
  var li = this.listePr
  var colPt = []
  for (var i = 0; i < arguments.length; i++) {
    var pt = arguments[i]
    colPt.push(new CNoeudPointeurSurPoint(li, pt))
  }
  var poly = new CPolygone(li, null, false, this.getCouleur(), false, this.getStyleTrait(), colPt)
  poly.positionne(false, this.dimf)
  if (!this.existeDeja(poly)) li.add(poly)
}

MtgApp.prototype.creeDefs = function () {
  // Création du linear  Gradiant pour les boutons
  var defs = kernel.cens('defs', {
    id: 'mtg32_defs'
  })
  var lg = kernel.cens('linearGradient', {
    id: 'arrowGrad',
    x1: '0%',
    y1: '50%',
    x2: '100%',
    y2: '50%'
  })
  let stop = kernel.cens('stop', {
    offset: '0%',
    style: 'stop-color:#0000FF;stop-opacity:0.6'
  })
  lg.appendChild(stop)
  stop = kernel.cens('stop', {
    offset: '100%',
    style: 'stop-color:#CEF6F5;stop-opacity:0.4'
  })
  lg.appendChild(stop)
  defs.appendChild(lg)

  var rg = kernel.cens('radialGradient', {
    id: 'buttonGrad',
    r: '100%'
  })
  stop = kernel.cens('stop', {
    offset: '0%',
    style: 'stop-color:#0000FF;stop-opacity:0'
  })
  rg.appendChild(stop)
  stop = kernel.cens('stop', {
    offset: '100%',
    style: 'stop-color:#0000FF;stop-opacity:0.2'
  })
  rg.appendChild(stop)
  defs.appendChild(rg)
  rg = kernel.cens('radialGradient', {
    id: 'buttonGradAct',
    r: '100%'
  })
  stop = kernel.cens('stop', {
    offset: '2%',
    style: 'stop-color:#0000FF;stop-opacity:0'
  })
  rg.appendChild(stop)
  stop = kernel.cens('stop', {
    offset: '100%',
    style: 'stop-color:#0000FF;stop-opacity:0.5'
  })
  rg.appendChild(stop)
  defs.appendChild(rg)
  // Création d'un radial gradiant pour les boutons des sliders
  rg = kernel.cens('radialGradient', {
    id: 'sliderGrad',
    r: '100%'
  })
  stop = kernel.cens('stop', {
    offset: '0%',
    style: 'stop-color:#9B9EFC;'
  })
  rg.appendChild(stop)
  stop = kernel.cens('stop', {
    offset: '100%',
    style: 'stop-color:#0000FF;'
  })
  rg.appendChild(stop)
  defs.appendChild(rg)
  this.svg.appendChild(defs)
  // un defs pour contenir les patterns de remplissages pour les surfaces avec quadrillage
  defs = kernel.cens('defs', {
    id: 'mtg32_patterns'
  })
  this.svg.appendChild(defs)
}

MtgApp.prototype.releaseSliders = function () {
  this.thicknessSlider.captured = false
  this.opacitySlider.captured = false
}

/**
 * Fonction vérifiant si le dernier élément de la liste est confondu avec un objet déjà créé
 * sachant que si c'est le cas il faut retirer les nbOjetsAjoutes derniers objets de la liste.
 * Si on ne trouve pas d'élément confondu, les nbObjetsAjoutes derniers objets sont positioonés
 * @param nbObjetsAjoutes
 */
MtgApp.prototype.verifieDernierElement = function (nbObjetsAjoutes) {
  var list = this.listePr
  var len = list.longueur()
  var der = list.get(len - 1)
  // Avant la vérification il faut positionner les derniers éléments ajoutés à la liste
  for (var i = 0; i < nbObjetsAjoutes; i++) {
    var ob = list.get(len - nbObjetsAjoutes + i)
    ob.positionne(false, this.dimf)
  }
  if (this.existeDeja(der)) {
    list.retireNDerniersElements(nbObjetsAjoutes)
    if (this.estExercice) this.listePourConst.retireNDerniersElements(nbObjetsAjoutes)
    this.nameEditor.montre(false)
    return false
  } else return true
}

MtgApp.prototype.detruitDependants = function (pt, bRemovegElements = true) {
  this.listePr.detruitDependants(pt, this.svgFigure, bRemovegElements)
  if (this.estExercice) this.listePourConst = this.listePourConstruction()
}

/**
 * Fonction détruisant les constructions itératives ou récursives de la figure dépendant du prototype prot
 * @param {CPrototype} proto
 */
MtgApp.prototype.detruitDepProto = function (proto) {
  var list = this.listePr
  for (let i = 0; i < list.longueur(); i++) {
    const el = list.get(i)
    if (el.depDeProto(proto)) this.detruitDependants(el, true)
  }
  this.reCreateDisplay()
}

/**
 * Fonction retirant toutes les définitions de quadraillages associés à des surfaces
 */
MtgApp.prototype.removeSurfacePatterns = function () {
  $('#mtg32_patterns').empty()
  var listePr = this.listePr
  for (var i = 0; i < listePr.longueur(); i++) {
    var el = listePr.get(i)
    if (el.estDeNature(NatObj.NSurface)) el.pattern = null
  }
}

/* Ne sert plus. Remplacé par une bonne gestion des événements souris ou touch pour que l'événement
   ne se propage pas.
   Finalement sert pour que quand on clique deux fois très rapidement sur ne boîte de dilogue avec une faute de
   syntaxe on n'ait deux dialogues d'avertissements ui s'emplilent
   */
MtgApp.prototype.lastDlgId = function () {
  if (this.dlg.length > 0) return this.dlg[this.dlg.length - 1]
  else return ''
}

MtgApp.prototype.showStopButton = function (bVisible) {
  $(this.stopButton.container).attr('visibility', bVisible ? 'visible' : 'hidden')
}

MtgApp.prototype.showInputs = function (bshow) {
  var list = this.listePr
  for (var i = 0; i < list.longueur(); i++) {
    var el = list.get(i)
    if (el.estDeNature(NatObj.NEditeurFormule) && el.existe && !el.masque) $(el.editeur).css('visibility', bshow ? 'visible' : 'hidden')
  }
}

/**
 * Fonction affichant en haut et à droite une message d'indication correspondant à getStr(ch);
 * Si le paramètre prech est présent, getStr(prech) est affiché devant getStr(ch) avec : entre les deux;
 * @param ch
 * @param prech
 * @param  {boolean} bstraight : Si true on n'utilise pas kernel.getStr(ch)
 */
MtgApp.prototype.indication = function (ch, prech = '', bstraight = false) {
  var st = bstraight ? ch : kernel.getStr(ch)
  if (st === '') return
  // Sur la version electron, quand on appuie sur F10 cela réaffiche la dernière indication
  this.lastindch = ch
  this.lastindprech = prech
  //
  if (prech.length !== 0) st = kernel.getStr(prech) + ' : ' + st
  this.effaceIndication()
  var CAlp = CAffLiePt
  this.comm = new CCommentaire(this.listePr, null, false, Color.blue, this.dimf.x - 10, 5, 0, 0, false, null,
    this.dys ? 20 : 15, StyleEncadrement.Simple, true, new Color(204, 204, 255), CAlp.alignHorRight, CAlp.alignVerTop,
    st, new CValeurAngle(this.listePr, 0))
  this.comm.positionne(false, this.dimf)
  this.comm.affiche(this.svgFigure, true, this.doc.couleurFond, this.doc.opacity)
  $(this.comm.g.childNodes[0]).attr('rx', 10)
  $(this.comm.g.childNodes[0]).attr('ry', 5)
  var self = this
  setTimeout(function () {
    if (self.comm !== null) {
      if (self.comm.chaineCommentaire === st) {
        try {
          self.comm.removegElement(self.svgFigure)
        } catch (e) {
        }
        self.comm = null
      }
    }
  }, 6000)
}

MtgApp.prototype.lastIndication = function () {
  this.indication(this.lastindch, this.lastindprech)
}

MtgApp.prototype.effaceIndication = function () {
  if ((this.comm !== null) && this.comm.g) {
    if (this.comm.g.parentNode === this.svgFigure) this.comm.removegElement(this.svgFigure)
  }
  this.comm = null
}

MtgApp.prototype.selectTool = function (toolName) {
  var name = 'outil' + toolName
  // Modification version 6.4.8 pour qu'on en puisse pas activer un outil si la figure a été bloquée
  // par un setAtive(fals)
  var activationPossible = (this.doc.isActive) || (toolName === 'Capt' || toolName === 'Protocole')
  if (activationPossible && this[name].activationValide()) {
    if (this[name].isSelectable) this.outilActif.deselect() // Par exemple les outils de zoom ne déscativent pas l'outil actif
    this[name].select()
    this.nameEditor.montre(false) // Des fois qu'un nom soit en cours d'édition
  }
}

MtgApp.prototype.activeOutilCapt = function () {
  // Modification version 6.3.3 pour pouvoir acirver directement l'outil précédent.
  this.outilActif.deselect()
  // this.outilActif = this.outilCapt;
  this.outilCapt.select()
}

// Ajout version 6.3.3
MtgApp.prototype.activeOutilPrec = function () {
  // Ne pas appeler si unen boîte de dialogue est ouverte !
  if ((this.dlg.length === 0) && this.outilActifPrec.activationValide()) {
    this.outilActif.deselect()
    this.outilActifPrec.select()
  }
}

/**
 * Fonction appelée pour la version electron par les raccourcis clavier
 * Elle acyive l'outil ainsi que son icône dans sa barre d'outils
 * @param toolName
 */
MtgApp.prototype.activateTool = function (toolName) {
  var name = 'outil' + toolName
  if (this[name].activationValide()) {
    this.nameEditor.montre(false) // Des fois qu'un nom soit en cours d'édition
    this.outilActif.deselect() // Par exemple les outils de zoom ne déscativent pas l'outil actif
    this[name].select()
    var row = 0
    for (var i = 0; i < this.expandableBars.length; i++) {
      var b = this.expandableBars[i]
      if (b.tools.length !== 0) {
        if (b.tools.indexOf(toolName) !== -1) {
          b.activeToolName = toolName
          b.updateActiveIcon(row)
          break
        }
        row++
      }
    }

    // this.unFoldExpandableBars();
    // this.updateActiveIcons();
  }
}

/**
 * Fonction renvoyant true si l'objet el a déjà été créé
 * S'il s'agit d'un exercice de construction on autorise la création d'un objet déjà présent
 * dans la figure au début de l'exercice.
 * @param el
 * @returns {boolean}
 */
MtgApp.prototype.existeDeja = function (el) {
  var list = this.listePr
  var res = false
  for (var i = this.estExercice ? this.nbObjInit : 0; (i < list.longueur()) && !res; i++) {
    var elb = list.get(i)
    // Modifié par rapport à la version Java car on peut vérifier un éléméent déjà ajouté à la liste
    // if (!elb.estElementIntermediaire()) res = res || elb.confonduAvec(el);
    if ((elb.className === el.className) && !elb.estElementIntermediaire() && (elb !== el)) { res = res || elb.confonduAvec(el) }
  }
  return res
}

MtgApp.prototype.copyFig = function () {
  this.getBase64ImageData('png', function (data) {
    copy(data) // eslint-disable-line no-undef
    // copy est défini dans index.html
  }, this.pref_coefMult, 1)
  new AvertDlg(this, kernel.getStr('CopyOK'))
}

MtgApp.prototype.reInitConst = function () {
  if (!this.estExercice) {
    this.listeSrcG.retireTout()
    this.listeSrcNG.retireTout()
    this.listeFinG.retireTout()
    this.listeFinNG.retireTout()
  }
}

/**
 * Fonction recréant un nouveau document
 */
MtgApp.prototype.resetDoc = function () {
  // var doc = new CMathGraphDoc(this.id + "figurePanel", true, true); // Modifié version 6.3.0
  var doc = new CMathGraphDoc(this.id, true, true)
  this.doc = doc
  // Ligne suivante nécessaire pour que les macros d'activation et désactivation du mode trace activent ou non
  // l'icône correspondante
  doc.app = this
  this.listePr = doc.listePr
  doc.dimf = this.dimf
  this.listePourConst = this.listePourConstruction()
}

/**
 * Fonction mettant à jour le curseur d'opacité
 */
MtgApp.prototype.updateOpacitySlider = function () {
  this.opacitySlider.update(Math.round(this.doc.opacity * 100))
}

MtgApp.prototype.getResult = function () {
  const ch = this.getBase64Code()
  let score
  if (this.estExercice) {
    if (this.validateAnswer()) score = 1
    else score = 0
  }
  return {
    fig: ch,
    score: score,
    level: this.levelIndex,
    dys: this.dys,
    nbObjInit: this.edition ? this.listePr.longueur() : this.nbObj
  }
}

/**
 * Fonction enlevant toutes les icpones de la barre d'outil supéreure et les remplaçant par des icônes adaptées
 * au document en cours
 */
MtgApp.prototype.updateToolbar = function () {
  var doc = this.doc
  var toolBar = this.toolBar
  var estExercice = this.estExercice
  // On détruit les icônes précédentes
  while (toolBar.childNodes.length !== 0) toolBar.removeChild(toolBar.childNodes[0])
  //
  var indOutil = 0
  this.buttonNew = new ButtonTool(this, 'New', 'top', false, this.newFig ? indOutil++ : -1)
  var b = !estExercice && window.File && window.FileReader && window.FileList && window.Blob
  this.buttonOpen = new ButtonTool(this, 'Open', 'top', false, this.open && b ? indOutil++ : -1)
  // this.buttonSave = new ButtonTool(this, "Save", "top", false, this.save && b ? indOutil++ : -1);
  this.buttonSave = new ButtonTool(this, 'Save', 'top', false, this.save ? indOutil++ : -1)
  this.buttonAnnuler = new ButtonTool(this, 'Annuler', 'top', false, indOutil++)
  this.buttonRefaire = new ButtonTool(this, 'Refaire', 'top', false, indOutil++)
  this.buttonSup = new ButtonTool(this, 'Sup', 'top', false, indOutil++)
  this.buttonModifObjGraph = new ButtonTool(this, 'ModifObjGraph', 'top', false, doc.toolDispo(this.outilModifObjGraph.toolIndex) ? indOutil++ : -1)
  this.buttonZoomPlus = new ButtonTool(this, 'ZoomPlus', 'top', false, doc.toolDispo(this.outilZoomPlus.toolIndex) ? indOutil++ : -1)
  this.buttonZoomMoins = new ButtonTool(this, 'ZoomMoins', 'top', false, doc.toolDispo(this.outilZoomMoins.toolIndex) ? indOutil++ : -1)
  this.buttonModifObjNum = new ButtonTool(this, 'ModifObjNum', 'top', false, doc.toolDispo(this.outilModifObjNum.toolIndex) ? indOutil++ : -1)
  this.buttonPalette = new ButtonTool(this, 'Palette', 'top', false, doc.toolDispo(this.outilPalette.toolIndex) ? indOutil++ : -1)
  this.buttonNommer = new ButtonTool(this, 'Nommer', 'top', false, indOutil++)
  this.buttonCaptNom = new ButtonTool(this, 'CaptNom', 'top', false, indOutil++)
  // En mode exercice on n'affiche pas les 3 boutons suivants mais on les crée pour qu'ils soient dispo dans le choix des options
  this.buttonModeTrace = new ButtonTool(this, 'ModeTrace', 'top', false, !estExercice && doc.toolDispo(this.outilModeTrace.toolIndex) ? indOutil++ : -1)
  this.buttonModePointsAuto = new ButtonTool(this, 'ModePointsAuto', 'top', false, !estExercice ? indOutil++ : -1)
  // this.buttonModePointsAuto.activate(this.pref_PointsAuto);
  this.buttonRecalculer = new ButtonTool(this, 'Recalculer', 'top', false, !estExercice && doc.toolDispo(this.outilRecalculer.toolIndex) ? indOutil++ : -1)
  // Fin
  this.buttonGomme = new ButtonTool(this, 'Gomme', 'top', false, doc.toolDispo(this.outilGomme.toolIndex) ? indOutil++ : -1)
  this.buttonRideau = new ButtonTool(this, 'Rideau', 'top', false, doc.toolDispo(this.outilRideau.toolIndex) ? indOutil++ : -1)
  this.buttonTranslationFigure = new ButtonTool(this, 'TranslationFigure', 'top', false, doc.toolDispo(this.outilTranslationFigure.toolIndex) ? indOutil++ : -1)
  this.buttonExecutionMacro = new ButtonTool(this, 'ExecutionMacro', 'top', false, doc.toolDispo(this.outilExecutionMacro.toolIndex) ? indOutil++ : -1)
  // En mode exercice on n'active pas le bouton d'exportation
  this.buttonExport = new ButtonTool(this, 'Export', 'top', false, !estExercice ? indOutil++ : -1)
  // this.buttonProtocole = new ButtonTool(this, "Protocole", "top", false, doc.toolDispo(this.outilProtocole.toolIndex) ? indOutil++ : -1);
  this.buttonProtocole = new ButtonTool(this, 'Protocole', 'top', false, this.modeBilan || doc.toolDispo(this.outilProtocole.toolIndex) ? indOutil++ : -1)
  // En mode exercice on n'active pas le bouton d'aide
  this.buttonHelp = new ButtonTool(this, 'Help', 'top', false, !estExercice ? indOutil++ : -1)
  this.buttonOptionsFig = new ButtonTool(this, 'OptionsFig', 'top', false, this.options ? indOutil++ : -1)
  // En mode exercice on n'affiche pas le bouton Outils supplémentaires
  this.buttonToggleToolsAdd = new ButtonTool(this, 'ToggleToolsAdd', 'top', false, !estExercice ? (doc.toolDispo(this.outilToggleToolsAdd.toolIndex) ? indOutil : -1) : -1)
}

/**
 * Fonctio utilisée pour les corrections d'exercices de construction sous j3P et n'ajoutant que l'outil de protocole
 * dans la barre d'outils horizontale.
 */
MtgApp.prototype.updateToolbarForCor = function () {
  var toolBar = this.toolBar
  // On détruit les icônes précédentes
  while (toolBar.childNodes.length !== 0) toolBar.removeChild(toolBar.childNodes[0])
  this.buttonProtocole = new ButtonTool(this, 'Protocole', 'top', false, 0)
}

/**
 * Fonction qui, si on a un exercice de construction, c'est-à dire une macro d'apparition d'initulé #Solution
 * et si le commentaire de cette macro commence par #Enonce, ce qui signifie que la figure contient un énoncé
 * qui doit être le dernier affichage de texte ou LaTeX de la figure initiale, renvoie un pointeur sur cet
 * affichage LaTeX.
 * Renvoie null s'il n'y en a pas.
 * On s'arrangera pour que le g element de cet affichage soit toujours le dernier à chaque action sur la figure.
 * @returns {*}
 */
MtgApp.prototype.getEnonce = function () {
  if (this.estExercice) {
    var mac = this.macroPourConst
    if (mac.commentaireMacro.indexOf('#Enonce') !== -1) {
      var list = this.listePr
      for (var i = list.longueur() - 1; i >= 0; i--) {
        var el = list.get(i)
        if (el.estDeNature(Nat.or(NatObj.NCommentaire, NatObj.NLatex))) {
          return el
        }
      }
      return null
    } else return null
  } else return null
}

/**
 * Fonction utilisée pour la correction des exerices de construction pour que le prof puisse capturer un point mobile
 * et voir comment la figure a été faite par l'élève
 */
MtgApp.prototype.activateForCor = function () {
  this.doc.setIdMenus(this, false, [33023, 32892], 3)
  this.updateToolsToolBar()
  this.updateToolbarForCor()
}

/**
 * Fonction qui, pour un exercice de construction, renvoie le nombre d'objets qu'a créés l'élève.
 * @returns {number}
 */
MtgApp.prototype.getNbObjConst = function () {
  if (this.estExercice) {
    var list = this.listePr
    var compt = 0
    for (var i = this.nbObjInit; i < list.longueur(); i++) {
      var el = list.get(i)
      if (el.estDeNature(NatObj.NTtObj) && !el.estElementIntermediaire() && !el.masque) compt++
    }
    return compt
  } else return 0
}

/**
 * Fonction ajoutée version 6.4 qui vérifie qu'il n'y a pas une macro en courus d'exécution et, si oui, la désactive.
 */
MtgApp.prototype.termineMacroEnCours = function () {
  const list = this.listePr
  const mac = list.macroEnCours
  if (mac !== null) {
    const macEnCours = mac.macroEnCours()
    const doc = this.doc
    if (macEnCours !== null) {
      list.terminerMacros = true // Utilisé dans CMacro.passageMacroSuivante
      macEnCours.termineAction(this.svgFigure, this.dimf, doc.couleurFond, doc.opacity)
      macEnCours.executionEnCours = false
      if (mac.macroLanceuse !== null) mac.macroLanceuse.termineAction(this.svgFigure, this.dimf, doc.couleurFond, doc.opacity)
    }
    list.macroEnCours = null
  }
}

/**
 * Fonction retitant les éventuels petits div associés à des variables en bas et à droite de la figure
 */
MtgApp.prototype.removePaneVariables = function () {
  this.listePr.removePaneVariables(this.svg.parentNode)
}
