/*!
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 * @version 5.7.0
 */
'use strict'

const loadDelay = 30 // en s (avec peu de débit faut bien 30s)

// pour le moment on charge mathjax en ligne, on verra plus tard pour
// http://docs.mathjax.org/en/latest/web/webpack.html#a-custom-mathjax-build
// (ce sera indispensable pour la version electron)
// const mathJaxUrl = 'https://cdn.jsdelivr.net/npm/mathjax@3/es5/tex-mml-svg.js'
const mathjax3Base = 'https://www.mathgraph32.org/js/MathJax3'
const mathJax3Uri = '/es5/tex-svg.js'

/* la config, on avait en v2 le js
https://www.mathgraph32.org/ftp/js/MathJax/MathJax.js?config=TeX-AMS-MML_SVG-full
avec la config :
 MathJax.Hub.Config({
  tex2jax: {
    inlineMath: [
      ['$', '$'],
      ['\\(', '\\)']
    ]
  },
  SVG: {
    mtextFontInherit: false
  },
  jax: ['input/TeX', 'output/SVG'],
  TeX: {
    extensions: ['color.js']
  },
  messageStyle: 'none'
})
qui donne, via https://mathjax.github.io/MathJax-demos-web/convert-configuration/convert-configuration.html
cette conf v3
 */
const mathJaxConfig = {
  tex: {
    inlineMath: [
      ['$', '$'],
      ['\\(', '\\)']
    ],
    // packages: {'[+]': ['noerrors', 'color']}
    packages: { '[+]': ['color'] }
  },
  svg: {
    // cf http://docs.mathjax.org/en/latest/options/output/svg.html#the-configuration-block
    mtextInheritFont: true
  },
  options: {
    ignoreHtmlClass: 'tex2jax_ignore',
    processHtmlClass: 'tex2jax_process'
  },
  loader: {
    // la conversion donnait ça
    // load: ['input/tex', 'output/svg']
    // mais ça marchait pas, c'est bcp mieux avec
    load: ['[tex]/noerrors', '[tex]/color']
  }
}

// si on fait deux appels très rapprochés à loadMathJax,
// on peut se retrouver dans le cas où le 2e retourne Promise.resolve() alors que MathJax n'est pas encore ready
let mathJaxLoadingPromise

function enhanceMathJax () {
  // pour la compatibilité on ajoute le MathJax.Hub
  // cf http://docs.mathjax.org/en/latest/upgrading/v2.html#version-2-compatibility-example
  MathJax.Callback = function (args) {
    if (Array.isArray(args)) {
      if (args.length === 1 && typeof (args[0]) === 'function') {
        return args[0]
      } else if (typeof (args[0]) === 'string' && args[1] instanceof Object &&
        typeof (args[1][args[0]]) === 'function') {
        return Function.bind.apply(args[1][args[0]], args.slice(1))
      } else if (typeof (args[0]) === 'function') {
        return Function.bind.apply(args[0], [window].concat(args.slice(1)))
      } else if (typeof (args[1]) === 'function') {
        return Function.bind.apply(args[1], [args[0]].concat(args.slice(2)))
      }
    } else if (typeof (args) === 'function') {
      return args
    }
    throw Error("Can't make callback from given data")
  } // MathJax.Callback

  MathJax.Hub = {
    Queue: function () {
      for (let i = 0, m = arguments.length; i < m; i++) {
        const fn = MathJax.Callback(arguments[i])
        MathJax.startup.promise = MathJax.startup.promise.then(fn)
      }
      // celui qui appelle Queue en v2 s'attend pas à récurpérer une promesse,
      // on gère donc les plantages en ajoutant le catch
      return MathJax.startup.promise.catch(error => console.error(error))
    },
    Typeset: () => console.error(Error('Il ne faut plus utiliser MathJax.Hub.Typeset en v3, remplacer par MathJax.typesetPromise() ou MathJax.typeset() (http://docs.mathjax.org/en/latest/upgrading/v2.html#changes-in-the-mathjax-api)'))
  } // MathJax.Hub
}

/**
 * Charge mathjax dans le <head> de la page courante
 * @param {Object} [options]
 * @param {string} [options.mathjax3Base] Un éventuel chemin vers le dossier de MathJax3 (qui devra contenir es5/tex-svg.js), sans slash de fin
 * @return {Promise}
 */
module.exports = function loadMathJax (options) {
  // si on a déjà été appelé y'a rien à faire (c'est peut-être encore en cours ou déjà résolu)
  if (mathJaxLoadingPromise) return mathJaxLoadingPromise

  // faut le charger
  mathJaxLoadingPromise = new Promise((resolve, reject) => {
    // si qqun d'autre a déjà chargé MathJax (iep dans un nœud j3p précédent par ex),
    // pas la peine de le refaire si c'est la bonne version
    if (typeof MathJax === 'object' && /^3\./.test(MathJax.version)) {
      // c'est du MathJax3, on vérifie qu'il a ce qu'on veut
      if (typeof MathJax.Callback !== 'function' || typeof MathJax.Hub !== 'object' || typeof MathJax.Hub.Queue !== 'function' || typeof MathJax.Hub.Typeset !== 'function') {
        // c'était un MathJax3 chargé par qqun d'autre
        enhanceMathJax()
      }
      // on force notre config, au cas où qqun l'aurait configuré différemment
      // cf http://docs.mathjax.org/en/latest/web/configuration.html#configuring-mathjax-after-it-is-loaded
      MathJax.config = mathJaxConfig
      // c'est tout bon
      return resolve()
    }

    // faut charger MathJax
    window.MathJax = mathJaxConfig
    // on ajoute la résolution de la promesse quand il sera prêt
    // cf http://docs.mathjax.org/en/latest/web/configuration.html#performing-actions-during-startup
    MathJax.startup = {
      ready: () => {
        // on vire le timeout
        clearTimeout(timeout)
        MathJax.startup.defaultReady()
        MathJax.startup.promise.then(() => {
          enhanceMathJax()
          resolve()
        })
      } // ready
    } // window.MathJax.startup

    // chargement mathjax
    const eltScript = document.createElement('script')
    eltScript.type = 'text/javascript'
    // sinon ff râle avec du :
    // La demande d’accès aux cookies ou au stockage sur « https://cdn.jsdelivr.net/npm/mathjax@3/es5/input/tex/extensions/noerrors.js » a été bloquée parce que nous bloquons toutes les demandes d’accès au stockage par des sites tiers et que le blocage de contenu est activé.
    // ce qui est curieux car il n'y a aucune demande…
    eltScript.crossOrigin = 'anonymous'
    eltScript.src = ((options && options.mathjax3Base) || window.mathjax3Base || mathjax3Base) + mathJax3Uri
    const head = document.getElementsByTagName('head')[0]
    head.appendChild(eltScript)
    // on ajoute un timeout de chargement
    const timeout = setTimeout(() => {
      reject(Error(`Mathjax non chargé après ${loadDelay}s d’attente`))
    }, loadDelay * 1000)
  })

  return mathJaxLoadingPromise
} // loadMathJax
