/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const NatCal = require('../types/NatCal')
const Ope = require('../types/Ope')
const CCalculAncetre = require('./CCalculAncetre')
const CalcR = require('../kernel/CalcR')
const CConstante = require('./CConstante')
const COp = require('./COperation')
const CFoncNVar = require('./CFoncNVar')
module.exports = CDeriveePartielle

// Attention : Ordre des 3 derniers paramètres inversés par rapport à version Java
/**
 * Classe représentant une dérivée partielle d'une fonction utilisateur à 2 ou 3 variables
 * par rapport à une de ses variables.
 * @constructor
 * @extends CFoncNVar
 * @param {CListeObjets} listeProprietaire  La liste propriétaire de l'objet.
 * @param {CImplementationProto} impProto  null ou la construction propriétaire de l'objet.
 * @param {boolean} estElementFinal  true si l'objet est un objet final de construction
 * @param {String} nomCalcul  Le nom de la fonction dérivée.
 * @param {CFonc} fonctionAssociee  La fonction dont c'est la dérivée.
 * @param {number} indiceVariableDerivation  Donne la variable par rapport à laquelle on dérivé (0 pour lapremière variable).
 * @param {CCb} calcul  Le calcul de la dérivée (peut être null).
 * @returns {CDeriveePartielle}
 */
function CDeriveePartielle (listeProprietaire, impProto, estElementFinal, nomCalcul,
  fonctionAssociee, indiceVariableDerivation, calcul) {
  if (arguments.length === 1) CFoncNVar.call(this, listeProprietaire)
  else {
    if (arguments.length <= 6) calcul = null
    // Ne pas appeler le constructeur de CFoncNVar car calcul peut être null
    /*
    CFoncNVar.call(this, listeProprietaire, impProto, estElementFinal, nomCalcul,
      fonctionAssociee.chaineVariable, calcul)
    ; */
    CCalculAncetre.call(this, listeProprietaire, impProto, estElementFinal, nomCalcul)
    this.fonctionAssociee = fonctionAssociee
    this.indiceVariableDerivation = indiceVariableDerivation
    this.calcul = calcul
    if (fonctionAssociee !== null) {
      this.nbVar = fonctionAssociee.nombreVariables()
      if (calcul === null) this.creeCalcul()
      this.chaineVariable = fonctionAssociee.chaineVariable
    }
  }
}
CDeriveePartielle.prototype = new CFoncNVar()
CDeriveePartielle.prototype.constructor = CDeriveePartielle
CDeriveePartielle.prototype.superClass = 'CFoncNVar'
CDeriveePartielle.prototype.className = 'CDeriveePartielle'

CDeriveePartielle.prototype.getClone = function (listeSource, listeCible) {
  var ind1 = listeSource.indexOf(this.impProto)
  var ind2 = listeSource.indexOf(this.fonctionAssociee)
  var ptelb = new CDeriveePartielle(listeCible, listeCible.get(ind1, 'CImplementationProto'),
    this.estElementFinal, this.nomCalcul, listeCible.get(ind2, 'CFoncNVar'), this.indiceVariableDerivation, null)
  if (listeCible.className !== 'CPrototype') ptelb.creeCalcul()
  return ptelb
}

CDeriveePartielle.prototype.nombreVariables = function () {
  return this.fonctionAssociee.nombreVariables()
}
CDeriveePartielle.prototype.reconstruitChaineCalcul = function () {
  this.chaineCalcul = ''
}
CDeriveePartielle.prototype.metAJour = function () {
  if (!this.fonctionAssociee.deriveePossible(this.indiceVariableDerivation)) {
    this.calcul = new COp(this.listeProprietaire, new CConstante(this.listeProprietaire, 1),
      new CConstante(this.listeProprietaire, 0), Ope.Divi)
  } else this.creeCalcul()
}
CDeriveePartielle.prototype.getNatureCalcul = function () {
  return NatCal.NDeriveePartielle
}
CDeriveePartielle.prototype.variableFormelle = function () {
  return this.fonctionAssociee.variableFormelle()
}
CDeriveePartielle.prototype.depDe = function (p) {
  if (this.elementTestePourDependDe === p) return this.dependDeElementTeste
  return this.memDep(CFoncNVar.prototype.depDe.call(this, p) || this.fonctionAssociee.depDe(p))
}
CDeriveePartielle.prototype.dependDePourBoucle = function (p) {
  return (p === this) || this.fonctionAssociee.dependDePourBoucle(p)
}
CDeriveePartielle.prototype.confonduAvec = function (p) {
  // CDeriveePartielle pt;
  if (p.className === this.className()) {
    // pt = (CDeriveePartielle) p;
    return (this.fonctionAssociee === p.fonctionAssociee) &&
    (this.indiceVariableDerivation === p.indiceVariableDerivation)
  } else return false
}
CDeriveePartielle.prototype.positionne = function (infoRandom, dimfen) {
  this.existe = this.fonctionAssociee.existe
}
/**
 * Fonction appelée pour les exercices de calcul et qui doit recalculer tous les éléments de la figure.
 * Donc ici il faut recalculer la formule de la dérivée
 * @returns {undefined}
 */
CDeriveePartielle.prototype.positionneFull = function () {
  this.existe = this.fonctionAssociee.existe
  if (this.existe) this.metAJour()
}

/**
 * Calcula la formule de la dérivée partielle et la met dans this.
 * @returns {undefined}
 */
CDeriveePartielle.prototype.creeCalcul = function () {
  this.calcul = CalcR.creeDerivee(this.fonctionAssociee.calcul, this.indiceVariableDerivation)
}
CDeriveePartielle.prototype.read = function (inps, list) {
  CFoncNVar.prototype.readSansCalcul.call(this, inps, list)
  var ind1 = inps.readInt()
  this.fonctionAssociee = list.get(ind1, 'CFoncNVar')
  this.nbVar = this.fonctionAssociee.nbVar
  this.indiceVariableDerivation = inps.readInt()
  if (list.classname !== 'CPrototype') {
    if (!this.fonctionAssociee.deriveePossible(this.indiceVariableDerivation)) {
      this.calcul = new COp(this.listeProprietaire, new CConstante(this.listeProprietaire, 1),
        new CConstante(this.listeProprietaire, 0), Ope.Divi)
    } else this.creeCalcul()
  }
}
CDeriveePartielle.prototype.write = function (oups, list) {
  CFoncNVar.prototype.writeSansCalcul.call(this, oups, list)
  var ind1 = list.indexOf(this.fonctionAssociee)
  oups.writeInt(ind1)
  oups.writeInt(this.indiceVariableDerivation)
}
