/**
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
import mathjs from '../kernel/mathjs'
const Complexe = require('../types/Complexe')
const Opef = require('../types/Opef')
const kernel = require('../kernel/kernel')
const CCb = require('./CCb')
const CCbGlob = require('../kernel/CCbGlob')
const Ope = require('../types/Ope')

// const matrix = require('matrix-js')
// module.exports = CFonction
export default CFonction

/**
 * Classe représentant dans un arbre binaire de calcul un appel de fonction réelle
 * à une variable prédéfinie.
 * @constructor
 * @extends CCb
 * @param {CListeObjets} listeProprietaire La liste propriétaire de l'objet
 * @param {Opef} opef Donne la nature de la fonction.
 * @param {CCb} operande Le calcul donnant l'opérande.
 * @returns {CFonction}
 */
function CFonction (listeProprietaire, opef, operande) {
  CCb.call(this, listeProprietaire)
  if (arguments.length !== 1) {
    this.opef = opef
    this.operande = operande
  }
  this.dejaPositionne = false // ne servira que pour un appel fonction Rand() de génération d'un nombre aléatoire.
  this.valeurDejaPrise = 0 // Comme son nom l'indique pour les appels de random
  this.valeurDejaPriseImaginaire = 0 // Partie imaginaire pour les fonctions complexes pour les appels de random
  this.z1loc = new Complexe()
}
CFonction.prototype = new CCb()
CFonction.prototype.constructor = CFonction
CFonction.prototype.superClass = 'CCb'
CFonction.prototype.className = 'CFonction'

CFonction.prototype.nature = function () {
  return CCbGlob.natFonction
}
CFonction.prototype.numeroVersion = function () {
  return 2
}
CFonction.prototype.nombreVariables = function () {
  return 1
}
CFonction.prototype.getClone = function (listeSource, listeCible) {
  var cloneOperande = this.operande.getClone(listeSource, listeCible)
  var ptcalc = new CFonction(listeCible, this.opef, cloneOperande)
  // Les trois lignes suivantes ajoutées version 4.4 pour que les valeurs rand ne changent pas dans l'annuler-refaire
  ptcalc.dejaPositionne = this.dejaPositionne
  ptcalc.valeurDejaPrise = this.valeurDejaPrise
  ptcalc.valeurDejaPriseImaginaire = this.valeurDejaPriseImaginaire
  return ptcalc
}
CFonction.prototype.initialisePourDependance = function () {
  CCb.prototype.initialisePourDependance.call(this)
  this.operande.initialisePourDependance()
}
CFonction.prototype.depDe = function (p) {
  if (this.elementTestePourDependDe === p) return this.dependDeElementTeste
  return this.memDep(CCb.prototype.depDe.call(this, p) || this.operande.depDe(p))
}
CFonction.prototype.dependDePourBoucle = function (p) {
  if (this.opef === Opef.Rand) {
    // if (p instanceof CVariableBornee) return true;
    if (p.className === 'CVariableBornee') return true
    else return this.operande.dependDePourBoucle(p)
  } else return this.operande.dependDePourBoucle(p)
}
CFonction.prototype.existe = function (infoRandom) {
  return this.operande.existe(infoRandom)
}
CFonction.prototype.resultatBase = function (infoRandom, x) {
  switch (this.opef) {
    case Opef.Abso:
      return Math.abs(x)
    case Opef.Enti:
      return Math.floor(x)
    case Opef.Raci:
    case Opef.Raci2:
      if (x >= 0) { return Math.sqrt(x) } else throw new Error(kernel.erreurCalculException)
    case Opef.Sinu:
      if (this.listeProprietaire.uniteAngle === kernel.uniteAngle.radian) return CCbGlob.sinus(x)
      else return CCbGlob.sinus(x * kernel.ConvDegRad)
    case Opef.Cosi:
      if (this.listeProprietaire.uniteAngle === kernel.uniteAngle.radian) return CCbGlob.cosinus(x)
      else return CCbGlob.cosinus(x * kernel.ConvDegRad)
    case Opef.Tang:
      if (this.listeProprietaire.uniteAngle === kernel.uniteAngle.radian) return CCbGlob.tangente(x)
      else {
        return CCbGlob.tangente(x * kernel.ConvDegRad)
      }
    case Opef.Logn:
      if (x > 0) return Math.log(x)
      else throw new Error(kernel.erreurCalculException)
    case Opef.Expo:
      return Math.exp(x)
    case Opef.ATan:
      if (this.listeProprietaire.uniteAngle === kernel.uniteAngle.radian) return Math.atan(x)
      else return Math.atan(x) * kernel.ConvRadDeg
    case Opef.ACos:
      if (this.listeProprietaire.uniteAngle === kernel.uniteAngle.radian) return Math.acos(x)
      else return Math.acos(x) * kernel.ConvRadDeg
    case Opef.ASin:
      if (this.listeProprietaire.uniteAngle === kernel.uniteAngle.radian) return Math.asin(x)
      else return Math.asin(x) * kernel.ConvRadDeg
    case Opef.Cosh:
      return (Math.exp(x) + Math.exp(-x)) / 2
    case Opef.Sinh:
      return (Math.exp(x) - Math.exp(-x)) / 2
    case Opef.Tanh:
      var a = Math.exp(x)
      var b = Math.exp(-x)
      return (a - b) / (a + b)
    case Opef.ArgCh:
      if (x >= 1) return Math.log(x + Math.sqrt(x * x - 1))
      else throw new Error(kernel.erreurCalculException)
    case Opef.ArgSh:
      return Math.log(x + Math.sqrt(x * x + 1))
    case Opef.ArgTh:
      if ((x > -1) && (x < 1)) return 0.5 * Math.log((1 + x) / (1 - x))
      else throw new Error(kernel.erreurCalculException)
    case Opef.Rand:
      if (infoRandom) {
        this.valeurDejaPrise = Math.random()
        this.dejaPositionne = true
        return this.valeurDejaPrise
      } else {
        if (!this.dejaPositionne) {
          this.valeurDejaPrise = Math.random()
          this.dejaPositionne = true
          return this.valeurDejaPrise
        } else return this.valeurDejaPrise
      }
    case Opef.fact:
      if ((x < 0) || (x !== Math.round(x))) throw new Error(kernel.erreurCalculException)
      if (x <= 32) {
        var p = 1
        for (var i = 2; i <= x; i++) {
          p = p * i
        }
        return p
      } else {
        var x1 = x
        var d = 1 + 1 / (12 * x1)
        x1 *= x
        d += 1 / (288 * x1)
        x1 *= x
        d -= 139 / (51840 * x1)
        x1 *= x
        d -= 571 / (2488320 * x1)
        x1 *= x
        d -= 163879 / (209018880 * x1)
        return Math.exp(0.5 * Math.log(2 * Math.PI * x) + x * (Math.log(x) - 1)) * d
      }
    case Opef.Transp: // Ajout version 6.7 pour la transposition d'une matrice
    case Opef.Deter: // Ajout version 6.7 pour la transposition d'une matrice
      // Appelé pour un réel ne fait rien, renvoie comme résultat le paramètre
      return x
    case Opef.Inv: // Ajouté version 6.7 pour inverse terme à terme d'une matrice
      if (x === 0) throw new Error(kernel.erreurCalculException)
      else return 1 / x
    default : throw new Error(kernel.erreurCalculException)
  }// switch Opef}
}
CFonction.prototype.resultat = function (infoRandom) {
  switch (this.opef) {
    case Opef.Left :
    case Opef.LeftC : // Rajouté version 4.7.2
      var mg = this.operande.membreGauche()
      if (mg === null) {
        throw new Error(kernel.erreurCalculException)
      } else return this.operande.membreGauche().resultat(infoRandom)
    case Opef.Right :
    case Opef.RightC : // Rajouté version 4.7.2
      var md = this.operande.membreDroit()
      if (md === null) {
        throw new Error(kernel.erreurCalculException)
      }
      return this.operande.membreDroit().resultat(infoRandom)
    default :
      var x1 = this.operande.resultat(infoRandom)
      return this.resultatBase(infoRandom, x1)
  }
}
CFonction.prototype.resultatFonction = function (infoRandom, valeurParametre) {
  switch (Opef) {
    case Opef.Left :
    case Opef.LeftC : // Rajouté version 4.7.2
      var mg = this.operande.membreGauche()
      if (mg === null) {
        throw new Error(kernel.erreurCalculException)
      }
      return this.operande.membreGauche().resultatFonction(infoRandom, valeurParametre)
    case Opef.Right :
    case Opef.RightC : // Rajouté version 4.7.2
      var md = this.operande.membreDroit()
      if (md === null) {
        throw new Error(kernel.erreurCalculException)
      }
      return this.operande.membreDroit().resultatFonction(infoRandom, valeurParametre)
    default :
      var x1 = this.operande.resultatFonction(infoRandom, valeurParametre)
      return this.resultatBase(infoRandom, x1)
  }
}

CFonction.prototype.resultatFonction = function (infoRandom, valeurParametre) {
  switch (this.opef) {
    case Opef.Left :
    case Opef.LeftC : // Rajouté version 4.7.2
      var mg = this.operande.membreGauche()
      if (mg === null) {
        throw new Error(kernel.erreurCalculException)
      }
      return this.operande.membreGauche().resultatFonction(infoRandom, valeurParametre)
    case Opef.Right :
    case Opef.RightC : // Rajouté version 4.7.2
      var md = this.operande.membreDroit()
      if (md === null) {
        throw new Error(kernel.erreurCalculException)
      }
      return this.operande.membreDroit().resultatFonction(infoRandom, valeurParametre)
    default :
      var x1 = this.operande.resultatFonction(infoRandom, valeurParametre)
      return this.resultatBase(infoRandom, x1)
  }
}
CFonction.prototype.resultatComplexeBase = function (infoRandom, z, zRes) {
  switch (this.opef) {
    case Opef.Abs:
      zRes.y = 0
      zRes.x = z.module()
      return
    case Opef.Abso :
      if (z.y !== 0) throw new Error(kernel.erreurCalculException)
      else {
        zRes.y = 0
        zRes.x = Math.abs(z.x)
        return
      }
    case Opef.Sin:
      z.sinComp(this.listeProprietaire.uniteAngle, zRes)
      return
    case Opef.Sinu :
      if (z.y !== 0) throw new Error(kernel.erreurCalculException)
      else {
        zRes.y = 0
        if (this.listeProprietaire.uniteAngle === kernel.uniteAngle.radian) zRes.x = CCbGlob.sinus(z.x)
        else zRes.x = CCbGlob.sinus(z.x * kernel.ConvDegRad)
        return
      }
    case Opef.Cos:
      z.cosComp(this.listeProprietaire.uniteAngle, zRes)
      return
    case Opef.Cosi :
      if (z.y !== 0) throw new Error(kernel.erreurCalculException)
      else {
        zRes.y = 0
        if (this.listeProprietaire.uniteAngle === kernel.uniteAngle.radian) zRes.x = CCbGlob.cosinus(z.x)
        else zRes.x = CCbGlob.cosinus(z.x * kernel.ConvDegRad)
        return
      }
    case Opef.Tan:
      z.tanComp(this.listeProprietaire.uniteAngle, zRes)
      return
    case Opef.Tang :
      if (z.y !== 0) throw new Error(kernel.erreurCalculException)
      else {
        zRes.y = 0
        if (this.listeProprietaire.uniteAngle === kernel.uniteAngle.radian) zRes.x = CCbGlob.tangente(z.x)
        else zRes.x = z.x * kernel.ConvDegRad // Corrigé version 6.7
        return
      }
    case Opef.Log:
      z.lnComp(this.listeProprietaire.uniteAngle, zRes)
      return
    case Opef.Logn :
      if ((z.y !== 0) || (z.x <= 0)) throw new Error(kernel.erreurCalculException)
      else {
        zRes.y = 0
        zRes.x = Math.log(z.x)
        return
      }
    case Opef.Exp:
      z.expComp(this.listeProprietaire.uniteAngle, zRes)
      return
    case Opef.Expo:
      if (z.y !== 0) throw new Error(kernel.erreurCalculException)
      else {
        zRes.y = 0
        zRes.x = Math.exp(z.x)
        return
      }
    case Opef.Ch:
      z.chComp(this.listeProprietaire.uniteAngle, zRes)
      return
    case Opef.Cosh:
      if (z.y !== 0) throw new Error(kernel.erreurCalculException)
      else {
        zRes.y = 0
        zRes.x = (Math.exp(z.x) + Math.exp(-z.x)) / 2
        return
      }
    case Opef.Sh:
      z.shComp(this.listeProprietaire.uniteAngle, zRes)
      return
    case Opef.Sinh :
      if (z.y !== 0) throw new Error(kernel.erreurCalculException)
      else {
        zRes.y = 0
        zRes.x = (Math.exp(z.x) - Math.exp(-z.x)) / 2
        return
      }
    case Opef.Th:
      z.thComp(this.listeProprietaire.uniteAngle, zRes)
      return
    case Opef.Tanh :
      if (z.y !== 0) throw new Error(kernel.erreurCalculException)
      else {
        zRes.y = 0
        var a = Math.exp(z.x)
        var b = Math.exp(-z.x)
        zRes.x = (a - b) / (a + b)
        return
      }
    case Opef.Rnd:
      zRes.y = 0
      if (infoRandom) {
        this.valeurDejaPrise = Math.random()
        this.dejaPositionne = true
        zRes.x = this.valeurDejaPrise
      } else {
        if (!this.dejaPositionne) {
          this.valeurDejaPrise = Math.random()
          this.dejaPositionne = true
          zRes.x = this.valeurDejaPrise
        } else zRes.x = this.valeurDejaPrise
      }
      break
    case Opef.Conj:
      zRes.x = z.x
      zRes.y = -z.y
      return
    case Opef.Arg:
      if ((z.x === 0) && (z.y === 0)) {
        zRes.x = 0
        zRes.y = 0
        throw new Error(kernel.erreurCalculException)
      } else {
        zRes.x = z.argument(this.listeProprietaire.uniteAngle)
        zRes.y = 0
        return
      }
    case Opef.Re:
      zRes.x = z.x
      zRes.y = 0
      return
    case Opef.Im:
      zRes.x = z.y
      zRes.y = 0
      return
    case Opef.Rac:
    case Opef.Rac2:
    case Opef.Raci:
    case Opef.Raci2:
      zRes.x = 0
      zRes.y = 0
      if ((z.y !== 0) || (z.x < 0)) throw new Error(kernel.erreurCalculException)
      else {
        zRes.x = Math.sqrt(z.x)
        return
      }
    case Opef.EntiC :
    case Opef.Enti :
      zRes.x = 0
      zRes.y = 0
      if (z.y !== 0) throw new Error(kernel.erreurCalculException)
      else {
        zRes.x = Math.floor(z.x)
        return
      }
    case Opef.ACosC :
    case Opef.ACos :
      // le résultat n'existe que si l'argument est réel
      if (z.y !== 0) throw new Error(kernel.erreurCalculException)
      else {
        zRes.y = 0
        if (this.listeProprietaire.uniteAngle === kernel.uniteAngle.radian) zRes.x = Math.acos(z.x)
        else zRes.x = Math.acos(z.x) * kernel.ConvRadDeg
        return
      }
    case Opef.ASinC :
    case Opef.ASin :
      // le résultat n'existe que si l'argument est réel
      if (z.y !== 0) throw new Error(kernel.erreurCalculException)
      else {
        zRes.y = 0
        if (this.listeProprietaire.uniteAngle === kernel.uniteAngle.radian) zRes.x = Math.asin(z.x)
        else zRes.x = Math.asin(z.x) * kernel.ConvRadDeg
        return
      }
    case Opef.ATanC :
    case Opef.ATan :
      // le résultat n'existe que si l'argument est réel
      if (z.y !== 0) throw new Error(kernel.erreurCalculException)
      else {
        zRes.y = 0
        if (this.listeProprietaire.uniteAngle === kernel.uniteAngle.radian) zRes.x = Math.atan(z.x)
        else zRes.x = Math.atan(z.x) * kernel.ConvRadDeg
        return
      }
    case Opef.ArgChC :
    case Opef.ArgCh :
      // le résultat n'existe que si l'argument est réel
      if (z.y !== 0) throw new Error(kernel.erreurCalculException)
      else {
        zRes.y = 0
        if (z.x >= 1) zRes.x = Math.log(z.x + Math.sqrt(z.x * z.x - 1))
        else throw new Error(kernel.erreurCalculException)
      }
      break
    case Opef.ArgShC :
    case Opef.ArgSh :
      // le résultat n'existe que si l'argument est réel
      if (z.y !== 0) throw new Error(kernel.erreurCalculException)
      else {
        zRes.y = 0
        zRes.x = Math.log(z.x + Math.sqrt(z.x * z.x + 1))
        return
      }
    case Opef.ArgThC :
      // le résultat n'existe que si l'argument est réel
      if (z.y !== 0) throw new Error(kernel.erreurCalculException)
      else {
        zRes.y = 0
        if ((z.x > -1) && (z.x < 1)) zRes.x = 0.5 * Math.log((1 + z.x) / (1 - z.x))
        else throw new Error(kernel.erreurCalculException)
      }
      break
    case Opef.factC:
    case Opef.fact:
      if (z.y !== 0) throw new Error(kernel.erreurCalculException)
      else {
        zRes.y = 0
        if ((z.x < 0) || (z.x !== Math.round(z.x))) throw new Error(kernel.erreurCalculException)
        if (z.x <= 32) {
          var p = 1
          for (var i = 2; i <= z.x; i++) {
            p = p * i
          }
          zRes.x = p
        } else {
          var x = z.x
          var x1 = x
          var d = 1 + 1 / (12 * x1)
          x1 *= x
          d += 1 / (288 * x1)
          x1 *= x
          d -= 139 / (51840 * x1)
          x1 *= x
          d -= 571 / (2488320 * x1)
          x1 *= x
          d -= 163879 / (209018880 * x1)
          zRes.x = Math.exp(0.5 * Math.log(2 * Math.PI * x) + x * (Math.log(x) - 1)) * d
        }
      }
  } // switch Opef}
}
CFonction.prototype.resultatComplexe = function (infoRandom, zRes) {
  switch (this.opef) {
    case Opef.Left :
    case Opef.LeftC : // Rajouté version 4.7.2
      this.operande.membreGauche().resultatComplexe(infoRandom, zRes)
      break // Rajouté version 4.7.2
    case Opef.Right :
    case Opef.RightC : // Rajouté version 4.7.2
      this.operande.membreDroit().resultatComplexe(infoRandom, zRes)
      break // Rajouté version 4.7.2
    default :
      this.operande.resultatComplexe(infoRandom, this.z1loc)
      this.resultatComplexeBase(infoRandom, this.z1loc, zRes)
  }
}
CFonction.prototype.resultatFonctionComplexe = function (infoRandom, valeurParametre, zRes) {
  switch (this.opef) {
    case Opef.Left :
    case Opef.LeftC : // Rajouté version 4.7.2
      var mg = this.operande.membreGauche()
      if (mg === null) throw new Error(kernel.erreurCalculException)
      mg.resultatFonctionComplexe(infoRandom, valeurParametre, zRes)
      break // Rajouté version 4.7.2
    case Opef.Right :
    case Opef.RightC : // Rajouté version 4.7.2
      var md = this.operande.membreDroit()
      if (md === null) throw new Error(kernel.erreurCalculException)
      md.resultatFonctionComplexe(infoRandom, valeurParametre, zRes)
      break // Rajouté version 4.7.2
    default :
      this.operande.resultatFonctionComplexe(infoRandom, valeurParametre, this.z1loc)
      this.resultatComplexeBase(infoRandom, this.z1loc, zRes)
  }
}
// Ajout version 6.7
CFonction.prototype.resultatMat = function (infoRandom) {
  const op = this.operande.resultatMat(infoRandom)
  if (this.opef === Opef.Frac) {
    // L'opérande ne peut être qu'un nombre, une matrice colonne ou une matrice ligne
    if (typeof op === 'number') {
      return mathjs.matrix([kernel.fracCont(op)])
    } else {
      const size = op.size()
      const n = size[0]
      const p = size[1]
      if (n > 1 && p > 1) throw new Error(kernel.erreurCalculException)
      if (n === 1) { // Matrice d'une ligne. ON renvoie une matrice de deux lignes
        const lig1 = []
        const lig2 = []
        for (let j = 0; j < p; j++) {
          // const nb = op(0, j)
          const nb = op.get([0, j])
          const frac = kernel.fracCont(nb)
          lig1.push(frac[0])
          lig2.push(frac[1])
        }
        return mathjs.matrix([lig1, lig2])
      } else { // Cas d'une matrice colonne
        const res = []
        for (let i = 0; i < n; i++) {
          // const nb = op(i, 0)
          const nb = op.egt([i, 0])
          const frac = kernel.fracCont(nb)
          res.push(frac)
        }
        return mathjs.matrix(res)
      }
    }
  }
  if (typeof op === 'number') return this.resultatBase(infoRandom, op)
  if (this.opef === Opef.Transp) {
    const size = op.size()
    const n = size[0]
    const p = size[1]
    const tab = []
    for (let j = 0; j < p; j++) {
      const lig = []
      tab.push(lig)
      for (let i = 0; i < n; i++) {
        lig.push(op.get([i, j]))
      }
    }
    return mathjs.matrix(tab)
  }
  if (this.opef === Opef.Deter) {
    const size = op.size()
    if (size[0] !== size[1]) throw new Error(kernel.erreurCalculException)
    // return op.det()
    return mathjs.det(op)
  }
  if (this.opef === Opef.Inv) { // Inverse terme à terme d'une matrice
    // On regarde si un des termes de la matrice est nul
    const size = op.size()
    const n = size[0]
    const p = size[1]
    const tab = []
    for (let i = 0; i < n; i++) {
      const lig = []
      tab.push(lig)
      for (let j = 0; j < p; j++) {
        const x = op.get([i, j])
        if (x === 0) throw new Error(kernel.erreurCalculException)
        lig.push(1 / x)
      }
    }
    return mathjs.matrix(tab)
  }
  // On applique la fonction à chacun des termes de la matrice
  const self = this
  // return matrix(op.map(x => self.resultatBase(infoRandom, x)))
  return op.map(function (value, index, matrix) {
    return self.resultatBase(infoRandom, value)
  })
}

CFonction.prototype.dependDeVariable = function (ind) {
  switch (this.opef) {
    case Opef.Left:
    case Opef.LeftC:
      return this.operande.membreGauche().dependDeVariable(ind)
    case Opef.Right:
    case Opef.RightC:
      return this.operande.membreDroit().dependDeVariable(ind)
    default:
      return this.operande.dependDeVariable(ind)
  }
}
CFonction.prototype.getCopie = function () {
  return new CFonction(this.listeProprietaire, this.opef, this.operande.getCopie())
}
CFonction.prototype.chaineCalcul = function (varFor) {
  var cha = this.operande.chaineCalculSansPar(varFor)
  var parouv = '('
  var parfer = ')'

  switch (this.opef) {
    case Opef.Abso:
    case Opef.Abs:
      return kernel.getStr('abs') + parouv + cha + parfer
    case Opef.Enti:
    case Opef.EntiC:
      return kernel.getStr('intgr') + parouv + cha + parfer
    case Opef.Raci:
    case Opef.Rac:
      return kernel.getStr('sqrt') + parouv + cha + parfer
    case Opef.Raci2:
    case Opef.Rac2:
      return kernel.getStr('rac') + parouv + cha + parfer
    case Opef.Sinu:
    case Opef.Sin:
      return kernel.getStr('sin') + parouv + cha + parfer
    case Opef.Cosi:
    case Opef.Cos:
      return kernel.getStr('cos') + parouv + cha + parfer
    case Opef.Tang:
    case Opef.Tan:
      return kernel.getStr('tan') + parouv + cha + parfer
    case Opef.Logn:
    case Opef.Log:
      return kernel.getStr('ln') + parouv + cha + parfer
    case Opef.Expo:
    case Opef.Exp:
      return kernel.getStr('exp') + parouv + cha + parfer
    case Opef.ATan:
    case Opef.ATanC:
      return kernel.getStr('arctan') + parouv + cha + parfer
    case Opef.ACos:
    case Opef.ACosC:
      return kernel.getStr('arccos') + parouv + cha + parfer
    case Opef.ASin:
    case Opef.ASinC:
      return kernel.getStr('arcsin') + parouv + cha + parfer
    case Opef.Cosh:
    case Opef.Ch:
      return kernel.getStr('ch') + parouv + cha + parfer
    case Opef.Sinh:
    case Opef.Sh:
      return kernel.getStr('sh') + parouv + cha + parfer
    case Opef.Tanh:
    case Opef.Th:
      return kernel.getStr('th') + parouv + cha + parfer
    case Opef.fact:
    case Opef.factC:
      return kernel.getStr('fact') + parouv + cha + parfer
    // Deux ajouts version 4.6
    case Opef.Left:
    case Opef.LeftC:
      return kernel.getStr('left') + parouv + cha + parfer
    case Opef.Right:
    case Opef.RightC:
      return kernel.getStr('right') + parouv + cha + parfer
    case Opef.Transp: // Ajout version 6.7 pour transposer une matrice
      return kernel.getStr('transp') + parouv + cha + parfer
    case Opef.Deter: // Ajout version 6.7 pour calculer le déterminant d'une matrice
      return kernel.getStr('deter') + parouv + cha + parfer
    case Opef.Inv: // Ajout version 6.7 pour calculer l'inverse d'une matrice
      return kernel.getStr('inv') + parouv + cha + parfer
    case Opef.Frac: // Ajout version 6.7 pour calculer l'inverse d'une matrice
      return kernel.getStr('frac') + parouv + cha + parfer
    // A revoir ici. Pour la version 1.5 j'ai oublie le ArgCh
    case Opef.ArgCh:
    case Opef.ArgChC:
      return kernel.getStr('argch') + parouv + cha + parfer
    case Opef.ArgSh:
    case Opef.ArgShC:
      return kernel.getStr('argsh') + parouv + cha + parfer
    case Opef.ArgTh:
    case Opef.ArgThC:
      return kernel.getStr('argth') + parouv + cha + parfer
    case Opef.Rand:
    case Opef.Rnd:
      return kernel.getStr('rand') + parouv + cha + parfer
    case Opef.Conj:
      return kernel.getStr('conj') + parouv + cha + parfer
    case Opef.Arg:
      return kernel.getStr('arg') + parouv + cha + parfer
    case Opef.Re:
      return kernel.getStr('re') + parouv + cha + parfer
    case Opef.Im:
      return kernel.getStr('im') + parouv + cha + parfer

    default:
      return ''
  } // switch Opef}}
}

// Avant la version 6.4.8, la foncton ci-dessous était CFonction.prototype.chaineLatexSans
// Modifié pour un meilleur rendu des conjugués
CFonction.prototype.chaineLatexSansPar = function (varFor) {
  const parouvLatex = '\\left('
  const parferLatex = '\\right)'
  const cha = this.operande.chaineLatexSansPar(varFor)

  switch (this.opef) {
    case Opef.Abso:
    case Opef.Abs:
      return '\\left| {' + cha + '} \\right|'
    case Opef.Enti:
    case Opef.EntiC:
      return kernel.getStr('intgr') + parouvLatex + cha + parferLatex
    case Opef.Raci:
    case Opef.Rac:
      return '\\sqrt{' + cha + '}'
    case Opef.Raci2:
    case Opef.Rac2:
      return '\\sqrt{' + cha + '}'
    case Opef.Sinu:
    case Opef.Sin:
      return '\\sin' + parouvLatex + cha + parferLatex
    case Opef.Cosi:
    case Opef.Cos:
      return '\\cos' + parouvLatex + cha + parferLatex
    case Opef.Tang:
    case Opef.Tan:
      return '\\tan' + parouvLatex + cha + parferLatex
    case Opef.Logn:
    case Opef.Log:
      return kernel.getStr('lnlat') + parouvLatex + cha + parferLatex
    case Opef.Expo:
    case Opef.Exp:
      return 'e^' + '{' + cha + '}'
    case Opef.ATan:
    case Opef.ATanC:
      return kernel.getStr('arctanlat') + parouvLatex + cha + parferLatex
    case Opef.ACos:
    case Opef.ACosC:
      return kernel.getStr('arccoslat') + parouvLatex + cha + parferLatex
    case Opef.ASin:
    case Opef.ASinC:
      return kernel.getStr('arcsinlat') + parouvLatex + cha + parferLatex
    case Opef.Cosh:
    case Opef.Ch:
      return kernel.getStr('chlat') + parouvLatex + cha + parferLatex
    case Opef.Sinh:
    case Opef.Sh:
      return kernel.getStr('shlat') + parouvLatex + cha + parferLatex
    case Opef.Tanh:
    case Opef.Th:
      return kernel.getStr('thlat') + parouvLatex + cha + parferLatex
    case Opef.fact:
    case Opef.factC:
      if ((this.operande.nature() & (CCbGlob.natIntegraleDansFormule | CCbGlob.natMoinsUnaire | CCbGlob.natOperation |
        CCbGlob.natSommeDansFormule)) !== 0) { return parouvLatex + cha + parferLatex + '!' } else return cha + '!' // A revoir
    // Deux ajouts version 4.6
    case Opef.Left:
    case Opef.LeftC:
      return kernel.getStr('Left') + parouvLatex + cha + parferLatex
    case Opef.Right:
    case Opef.RightC:
      return kernel.getStr('Left') + parouvLatex + cha + parferLatex
    case Opef.Transp: // Ajout version 6.7 pour transposer une matrice
      return kernel.getStr('transp') + parouvLatex + cha + parferLatex
    case Opef.Deter: // Ajout version 6.7 pour calculer le déterminant d'une matrice
      return kernel.getStr('deter') + parouvLatex + cha + parferLatex
    case Opef.Inv: // Ajout version 6.7 pour calculer l'inverse d'une matrice
      return kernel.getStr('inv') + parouvLatex + cha + parferLatex
    case Opef.Frac: // Ajout version 6.7 pour calculer l'inverse d'une matrice
      return kernel.getStr('frac') + parouvLatex + cha + parferLatex
    // A revoir ici. Pour la version 1.5 j'ai oublie le ArgCh
    case Opef.ArgCh:
    case Opef.ArgChC:
      return kernel.getStr('argch') + parouvLatex + cha + parferLatex
    case Opef.ArgSh:
    case Opef.ArgShC:
      return kernel.getStr('argsh') + parouvLatex + cha + parferLatex
    case Opef.ArgTh:
    case Opef.ArgThC:
      return kernel.getStr('argth') + parouvLatex + cha + parferLatex
    case Opef.Rand:
    case Opef.Rnd:
      return kernel.getStr('rand') + parouvLatex + cha + parferLatex
    case Opef.Conj:
    {
      // Version 6.4.8 : On rajoute un petit espace devant un conjugué
      // return '\\overline {' + cha + '}'
      const op = this.operande
      if ((op.nature() === CCbGlob.natOperation) && (op.ope === Ope.Divi)) {
        return '\\,\\overline {' + parouvLatex + op.chaineLatexSansPar(varFor) + parferLatex + '}'
      } else return '\\,\\overline {' + cha + '}'
    }
    case Opef.Arg:
      return '\\arg' + parouvLatex + cha + parferLatex
    case Opef.Re:
      return kernel.getStr('re') + parouvLatex + cha + parferLatex
    case Opef.Im:
      return kernel.getStr('im') + parouvLatex + cha + parferLatex

    default:
      return ''
  } // switch Opef}}
}

// Fonction ajoutée version 6.4.8 pour un meilleur rendu des conjugues
CFonction.prototype.chaineLatex = function (varFor) {
  const parouvLatex = '\\left('
  const parferLatex = '\\right)'
  const op = this.operande
  if (this.opef === Opef.Conj) {
    if (op.nature() === CCbGlob.natOperation) {
      if (op.ope === Ope.Mult) return this.chaineLatexSansPar(varFor)
      else {
        if (op.ope === Ope.Divi) {
          return '\\,\\overline {' + parouvLatex + op.chaineLatexSansPar(varFor) + parferLatex + '}'
        } else return '\\,' + parouvLatex + '\\overline {' + op.chaineLatexSansPar(varFor) + '}' + parferLatex
      }
    } else return this.chaineLatexSansPar(varFor)
  } else return this.chaineLatexSansPar(varFor)
}

CFonction.prototype.read = function (inps, list) {
  CCb.prototype.read.call(this, inps, list)
  this.opef = inps.readByte()
  // A partir de la version 3.0, décalage des indices de fonctions complexes
  if ((this.nVersion === 1) && (this.opef >= 19)) this.opef += 45
  this.operande = inps.readObject(list)
  if (this.opef === Opef.Rand) {
    this.valeurDejaPrise = inps.readDouble()
    this.dejaPositionne = true
  }
}
CFonction.prototype.write = function (oups, list) {
  CCb.prototype.write.call(this, oups, list)
  oups.writeByte(this.opef)
  oups.writeObject(this.operande)
  if (this.opef === Opef.Rand) {
    oups.writeDouble(this.valeurDejaPrise)
  }
}
CFonction.prototype.estConstant = function () {
  return (this.opef !== Opef.Rand) && this.operande.estConstant()
}
CFonction.prototype.deriveePossible = function (indiceVariable) {
  return this.operande.deriveePossible(indiceVariable)
}
CFonction.prototype.calculAvecValeursRemplacees = function (bfrac) {
  return new CFonction(this.listeProprietaire, this.opef, this.operande.calculAvecValeursRemplacees(bfrac))
}
// Déplacé ici version 6.4.7
CFonction.prototype.calculNormalise = function (rempval, sousdivnorm, rempDecParFrac) {
  switch (this.opef) {
    case Opef.Left:
    case Opef.LeftC:
      return this.operande.membreGauche().calculNormalise(rempval, sousdivnorm, rempDecParFrac)
    case Opef.Right:
    case Opef.RightC:
      return this.operande.membreDroit().calculNormalise(rempval, sousdivnorm, rempDecParFrac)
    default :
      return new CFonction(this.listeProprietaire, this.opef, this.operande.calculNormalise(rempval, sousdivnorm, rempDecParFrac))
  }
}
