/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const StyleFleche = require('../types/StyleFleche')
const Vect = require('../types/Vect')
const kernel = require('../kernel/kernel')
const CMarqueAngleGeometrique = require('./CMarqueAngleGeometrique')
const CValDyn = require('../objets/CValDyn')
module.exports = CMarqueAngleOriente

/**
 * Classe représentant une marque d'angle orienté.
 * @constructor
 * @extends CMarqueAngleGeometrique
 * @param {CListeObjets} listeProprietaire  La liste propriétaire.
 * @param {CImplementationProto} impProto  null ou la construction propriétaire.
 * @param {boolean} estElementFinal  true si élément final de construction.
 * @param {Color} couleur  La couleur.
 * @param {boolean} masque  true si l'objet est masqué.
 * @param {StyleTrait} style  Donne le style trait utilisé.
 * @param {StyleMarqueAngle} styleMarque  Donne le style de la marque.
 * @param {number} rayon  Le rayon de l'arc.
 * @param {CPt} a  Le premier point
 * @param {CPt} o  Le point sommet de l'angle définissant l'arc.
 * @param {CPt} b  Le troisième point.
 * @param {StyleFleche} motifFleche  Le style utilisé pour la flèche finissant l'arc.
 * @returns {CMarqueAngleOriente}
 */
function CMarqueAngleOriente (listeProprietaire, impProto, estElementFinal, couleur, masque,
  style, styleMarque, rayon, a, o, b, motifFleche) {
  if (arguments.length === 0) return // Ajout version 4.9.9.4
  if (arguments.length === 1) CMarqueAngleGeometrique.call(this, listeProprietaire)
  else {
    CMarqueAngleGeometrique.call(this, listeProprietaire, impProto, estElementFinal, couleur,
      masque, style, styleMarque, rayon, a, o, b)
    this.motifFleche = motifFleche
  }
  this.xdeb = 0 // Les coordonnées du point de début de l'arc
  this.ydeb = 0
  this.v0 = new Vect() // Vect de longueur rayon colinéaire à OB
  this.vecteurDirection = new Vect() // Vect définissant la direction inverse de l'arc
}
CMarqueAngleOriente.prototype = new CMarqueAngleGeometrique()
CMarqueAngleOriente.prototype.constructor = CMarqueAngleOriente
CMarqueAngleOriente.prototype.superClass = 'CMarqueAngleGeometrique'
CMarqueAngleOriente.prototype.className = 'CMarqueAngleOriente'

CMarqueAngleOriente.prototype.getClone = function (listeSource, listeCible) {
  var ind1 = listeSource.indexOf(this.a)
  var ind2 = listeSource.indexOf(this.o)
  var ind3 = listeSource.indexOf(this.b)
  var ind4 = listeSource.indexOf(this.impProto)
  return new CMarqueAngleOriente(listeCible, listeCible.get(ind4, 'CImplementationProto'),
    this.estElementFinal, this.couleur, this.masque, this.style.getClone(), this.styleMarque, this.rayon,
    listeCible.get(ind1, 'CPt'), listeCible.get(ind2, 'CPt'),
    listeCible.get(ind3, 'CPt'), this.motifFleche)
}
CMarqueAngleOriente.prototype.setClone = function (ptel) {
  CMarqueAngleGeometrique.prototype.setClone.call(this, ptel)
  ptel.vecteurDirection.setCopy(this.vecteurDirection)
  this.xdeb = ptel.xdeb
  this.ydeb = ptel.ydeb
}
CMarqueAngleOriente.prototype.positionne = function (infoRandom, dimfen) {
  var petitAngle // Représente l'angle de correction pour la direction de la flèche
  // due à la longueur de la flèche
  CMarqueAngleGeometrique.prototype.positionne.call(this, infoRandom, dimfen)
  if (!this.existe) return
  // Optimisation pour la version 1.9.5
  var longueurFleche = StyleFleche.longeurFlechePourArc(this.motifFleche, this.listeProprietaire)
  this.v.vecteurColineaire(this.rayon, this.v0)
  var angle = this.u.mesureAngleVecteurs(this.v)
  if (kernel.zeroAngle(Math.abs(angle) - Math.PI / 2)) petitAngle = 0
  // Modifié version 5.3. Meilleur quand la marque est petite
  // else petitAngle = Math.atan(longueurFleche/this.rayon);
  else petitAngle = Math.atan(longueurFleche / this.rayon) / 2
  if (kernel.zero(angle)) {
    this.existe = false
    return
  } else {
    if (kernel.zeroAngle(Math.abs(angle) - Math.PI)) this.v0.tourne(-Math.PI / 2, this.vecteurDirection)
    else {
      if (angle <= 0) this.v0.tourne(Math.PI / 2 + petitAngle, this.vecteurDirection)
      else this.v0.tourne(-Math.PI / 2 - petitAngle, this.vecteurDirection)
    }
  }
  this.xdeb = this.o.x + this.v0.x
  this.ydeb = this.o.y + this.v0.y
}
CMarqueAngleOriente.prototype.createg = function (svg, couleurFond, transparence) {
  var g = CMarqueAngleGeometrique.prototype.createg.call(this, svg, couleurFond, transparence)
  g.appendChild(this.creeFlechePourArc())
  // Ligne suivante modifiée version 6.5.2
  // g.setAttribute('pointer-events', 'none')
  g.setAttribute('pointer-events', this.pointerevents)
  return g
}
// Procédure dessinant la flèche en bout d'une marque d'angle orienté
// Le Vect u désigne la direction opposée à celle de la flèche
// et (xa, ya) le point extrémité de la flèche
/**
 * Fonction créant le svg element représentant la flèche de l'arc.
 * Le Vect u désigne la direction opposée à celle de la flèche
 * et (xa, ya) le point extrémité de la flèche.
 * @returns {svg.Element}
 */
CMarqueAngleOriente.prototype.creeFlechePourArc = function () {
  var abs0, ord0, abs2, ord2, u1, u2, longueurFleche
  var path = kernel.cens('path')
  var style = ''
  var stroke = this.style.stroke
  if (stroke.length !== 0) style += 'stroke-dasharray:' + stroke + ';'
  var strokewidth = this.style.strokeWidth
  style += 'stroke-width:' + strokewidth + ';'
  // style += "stroke:"+this.couleur.rgb()+";"; Modifié version 4.9.9.4
  style += 'stroke:' + this.color + ';'
  u1 = new Vect()
  u2 = new Vect()
  longueurFleche = StyleFleche.longeurFlechePourArc(this.motifFleche, this.listeProprietaire)
  this.vecteurDirection.vecteurColineaire(longueurFleche, u1)
  // On fait tourner le Vect u1 de -pi/6 pour avoir u2
  u2.x = u1.x * kernel.cos30 - u1.y * kernel.sin30
  u2.y = u1.x * kernel.sin30 + u1.y * kernel.cos30
  abs0 = this.xdeb + u2.x
  ord0 = this.ydeb + u2.y
  // On fait tourner le Vect u1 de pi/6 pour avoir u2
  u2.x = u1.x * kernel.cos30 + u1.y * kernel.sin30
  u2.y = -u1.x * kernel.sin30 + u1.y * kernel.cos30
  abs2 = this.xdeb + u2.x
  ord2 = this.ydeb + u2.y
  switch (this.motifFleche) {
    case StyleFleche.FlecheLonguePleine:
    case StyleFleche.FlecheCourtePleine:
      // style += "fill:"+this.couleur.rgb()+";"; Modifié version 4.9.9.4
      style += 'fill:' + this.color + ';'
      path.setAttribute('d', 'M' + abs2 + ' ' + ord2 + 'L' + this.xdeb + ' ' + this.ydeb + 'L' + abs0 + ' ' + ord0 + 'Z')
      break
    case StyleFleche.FlecheLongueSimple:
    case StyleFleche.FlecheCourteSimple:
      style += 'fill:none;'
      path.setAttribute('d', 'M' + abs2 + ' ' + ord2 + 'L' + this.xdeb + ' ' + this.ydeb + 'L' + abs0 + ' ' + ord0)
  }
  path.setAttribute('style', style)
  return path
}
/**
 *
 * Renvoie true si le nom de la valeur commence par la chaîne st
 * @param st
 * @returns {boolean}
 */
CMarqueAngleOriente.prototype.nomCommencePar = function (st) {
  if (CValDyn.prototype.nomCommencePar.call(this, st)) return true
  var ch = this.point2.nom + this.point1.nom
  return ch.indexOf(st) === 0
}

CMarqueAngleOriente.prototype.read = function (inps, list) {
  CMarqueAngleGeometrique.prototype.read.call(this, inps, list)
  this.motifFleche = inps.readByte()
}
CMarqueAngleOriente.prototype.write = function (oups, list) {
  CMarqueAngleGeometrique.prototype.write.call(this, oups, list)
  oups.writeByte(this.motifFleche)
}
