/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
import mathjs from '../kernel/mathjs'
const Complexe = require('../types/Complexe')
const Fonte = require('../types/Fonte')
const Ope = require('../types/Ope')
const kernel = require('../kernel/kernel')
const CCb = require('./CCb')
const CCbGlob = require('../kernel/CCbGlob')
// const matrix = require('matrix-js')

// module.exports = COperation
export default COperation

/**
 * @constructor
 * @extends CCb
 * Classe représentant une opération dans un arbre binaire de calcul.
 * @param {CListeObjets} listeProprietaire  La liste propriétaire
 * @param {CCb} operande1  L'opérande de gauche
 * @param {CCb} operande2  L'opérande de droite
 * @param {number} ope  Caractérise l'opérateur. Voir Operateur.
 * @returns {COperation}
 */
function COperation (listeProprietaire, operande1, operande2, ope) {
  CCb.call(this, listeProprietaire)
  if (arguments.length === 1) CCb.call(this, listeProprietaire)
  else {
    this.operande1 = operande1
    this.operande2 = operande2
    this.ope = ope
  }
  this.c1 = new Complexe()
  this.c2 = new Complexe()
}
COperation.prototype = new CCb()
COperation.prototype.constructor = COperation
COperation.prototype.superClass = 'CCb'
COperation.prototype.className = 'COperation'

COperation.zeroComp = '0+0*i'

COperation.prototype.nature = function () {
  return CCbGlob.natOperation
}

COperation.prototype.getClone = function (listeSource, listeCible) {
  var cloneOperande1 = this.operande1.getClone(listeSource, listeCible)
  var cloneOperande2 = this.operande2.getClone(listeSource, listeCible)
  return new COperation(listeCible, cloneOperande1, cloneOperande2, this.ope)
}

COperation.prototype.initialisePourDependance = function () {
  CCb.prototype.initialisePourDependance.call(this)
  this.operande1.initialisePourDependance()
  this.operande2.initialisePourDependance()
}

COperation.prototype.depDe = function (p) {
  return this.memDep(this.operande1.depDe(p) || this.operande2.depDe(p))
}

COperation.prototype.dependDePourBoucle = function (p) {
  return this.operande1.dependDePourBoucle(p) || this.operande2.dependDePourBoucle(p)
}

COperation.prototype.existe = function (infoRandom) {
  return this.operande1.existe(infoRandom) && this.operande2.existe(infoRandom)
}
/**
 * Fonction renvoyant le résultat de l'opérateur appliqué aux nombres réels
 * op1 (opérande de gauche) et op2 (opérande de droite).
 * @param {number} op1  Opérande de gauche
 * @param {number} op2  Opérande de droite
 * @returns {number} : Le résultat de l'opération
 */
COperation.prototype.resultatBase = function (op1, op2) {
  var denom
  switch (this.ope) {
    case Ope.Plus:
      return op1 + op2
    case Ope.Moin:
      return op1 - op2
    case Ope.Mult:
      return op1 * op2
    // à revoir pour gestion des exceptions
    case Ope.Divi:
      denom = op2
      if (denom === 0) throw new Error(kernel.erreurCalculException)
      else return op1 / denom
    case Ope.Inf:
      if (op1 < op2) { return 1 } else return 0
    case Ope.Sup:
      if (op1 > op2) { return 1 } else return 0
    case Ope.InfOuEgal:
      if (op1 <= op2) { return 1 } else return 0
    case Ope.SupOuEgal:
      if (op1 >= op2) { return 1 } else return 0
    case Ope.Egalite:
      if (op1 === op2) { return 1 } else return 0
    case Ope.Diff:
      if (op1 !== op2) { return 1 } else return 0
    // Ajout version 4.9.5
    case Ope.And:
      if ((op1 === 1) && (op2 === 1)) return 1; else return 0
    case Ope.Or:
      if ((op1 === 1) || (op2 === 1)) return 1; else return 0

    default: return 0
  } // switch Ope
}

COperation.prototype.resultat = function (infoRandom) {
  var o1 = this.operande1.resultat(infoRandom)
  var o2 = this.operande2.resultat(infoRandom)
  return this.resultatBase(o1, o2)
}
COperation.prototype.resultatFonction = function (infoRandom, valeurParametre) {
  var o1 = this.operande1.resultatFonction(infoRandom, valeurParametre)
  var o2 = this.operande2.resultatFonction(infoRandom, valeurParametre)
  return this.resultatBase(o1, o2)
}
// op1 et op2 pointent sur deux complexes : les opérandes
/**
 * Fonction renvoyant le résultat de l'opérateur appliqué aux nombres complexes
 * pointés par op1 (opérande de gauche) et op2 (opérande de droite).
 * @param {Complexe} op1  Opérande de gauche
 * @param {Complexe} op2  Opérande de droite
 * @param {type} zRes  Le complexe dans lequel on met le résultat de l'opération.
 * @returns {undefined}
 */
COperation.prototype.resultatComplexeBase = function (op1, op2, zRes) {
  switch (this.ope) {
    case Ope.Plus:
      zRes.x = op1.x + op2.x
      zRes.y = op1.y + op2.y
      return
    case Ope.Moin:
      zRes.x = op1.x - op2.x
      zRes.y = op1.y - op2.y
      return
    case Ope.Mult:
      zRes.x = op1.x * op2.x - op1.y * op2.y
      zRes.y = op1.x * op2.y + op1.y * op2.x
      return
    case Ope.Divi:
      if ((op2.x === 0) && (op2.y === 0)) throw new Error(kernel.erreurCalculException)
      else kernel.divComp(op1, op2, zRes)
      return
    case Ope.Inf:
      // Si on est dans C, on n'accepte de comparer deux complexes
      // que s'ils sont réels
      zRes.x = 0
      zRes.y = 0
      if ((op1.y !== 0) || (op2.y !== 0)) { throw new Error(kernel.erreurCalculException) } else {
        var mod1 = op1.x * op1.x + op1.y * op1.y
        var mod2 = op2.x * op2.x + op2.y * op2.y
        if (mod1 < mod2) zRes.x = 1
        else zRes.x = 0
        return
      }
    case Ope.Sup:
      // Si on est dans C, on n'accepte de comparer deux complexes
      // que s'ils sont réels
      zRes.x = 0
      zRes.y = 0
      if ((op1.y !== 0) || (op2.y !== 0)) { throw new Error(kernel.erreurCalculException) } else {
        if (op1.x > op2.x) zRes.x = 1
        else zRes.x = 0
        return
      }
    case Ope.InfOuEgal:
      // Si on est dans C, on n'accepte de comparer deux complexes
      // que s'ils sont réels
      zRes.x = 0
      zRes.y = 0
      if ((op1.y !== 0) || (op2.y !== 0)) { throw new Error(kernel.erreurCalculException) } else {
        if (op1.x <= op2.x) zRes.x = 1
        else zRes.x = 0
        return
      }
    case Ope.SupOuEgal:
      // Si on est dans C, on n'accepte de comparer deux complexes
      // que s'ils sont réels
      zRes.x = 0
      zRes.y = 0
      if ((op1.y !== 0) || (op2.y !== 0)) throw new Error(kernel.erreurCalculException)
      else {
        if (op1.x >= op2.x) zRes.x = 1
        else zRes.x = 0
        return
      }
    case Ope.Egalite:
      zRes.y = 0
      if ((op1.x === op2.x) && (op1.y === op2.y)) zRes.x = 1
      else zRes.x = 0
      return
    case Ope.Diff:
      zRes.y = 0
      if ((op1.x !== op2.x) || (op1.y !== op2.y)) zRes.x = 1
      else zRes.x = 0
      return
    // Ajout version 4.9.5
    case Ope.And:
      zRes.y = 0
      if ((op1.y !== 0) || (op2.y !== 0)) zRes.x = 0 // aux si les opérandes ne sont pas réels
      else {
        if ((op1.x === 1) && (op2.x === 1)) zRes.x = 1
        else zRes.x = 0
      }
      return
    case Ope.Or:
      zRes.y = 0
      if ((op1.y !== 0) || (op2.y !== 0)) zRes.x = 0 // Faux si les opérandes ne sont pas réels
      else {
        if ((op1.x === 1) || (op2.x === 1)) zRes.x = 1
        else zRes.x = 0
      }
  } // switch Ope
}

COperation.prototype.resultatComplexe = function (infoRandom, zRes) {
  this.operande1.resultatComplexe(infoRandom, this.c1)
  this.operande2.resultatComplexe(infoRandom, this.c2)
  this.resultatComplexeBase(this.c1, this.c2, zRes)
}

COperation.prototype.resultatFonctionComplexe = function (infoRandom, valeurParametre, zRes) {
  this.operande1.resultatFonctionComplexe(infoRandom, valeurParametre, this.c1)
  this.operande2.resultatFonctionComplexe(infoRandom, valeurParametre, this.c2)
  this.resultatComplexeBase(this.c1, this.c2, zRes)
}

COperation.prototype.resultatMat = function (infoRandom) {
  var op1 = this.operande1.resultatMat(infoRandom)
  var op2 = this.operande2.resultatMat(infoRandom)
  if (typeof op1 === 'number' && typeof op2 === 'number') return this.resultatBase(op1, op2)
  if (typeof op1 === 'number') {
    // Cas du produit d'une matrice par un nombre
    return this.applyToMat(op1, op2, true)
  }
  if (typeof op2 === 'number') {
    // Cas du produit d'une matrice par un nombre
    return this.applyToMat(op2, op1, false)
  }
  // Cas du produit de deux matrices qui sont données sous forme d'arrays
  const size1 = op1.size()
  const n1 = size1[0] // Nb lignes matrice de droite
  const p1 = size1[1] // Nb colonnes matrice de gauche
  const size2 = op2.size()
  const n2 = size2[0] // Nb lignes matrice de droite
  const p2 = size2[1] // Nb colonnes matrice de droite
  let res, size
  switch (this.ope) {
    case Ope.Mult :
      if (p1 !== n2) throw new Error(kernel.erreurCalculException)
      res = mathjs.mult(op1, op2)
      size = res.size()
      if (size[0] === 1 && size[1] === 1) return res.get([0, 0])
      return res
    case Ope.Plus :
      if (n1 !== n2 || p1 !== p2) throw new Error(kernel.erreurCalculException)
      res = mathjs.add(op1, op2)
      size = res.size()
      if (size[0] === 1 && size[1] === 1) return res.get([0, 0])
      return res
    case Ope.Moin :
      if (n1 !== n2 || p1 !== p2) throw new Error(kernel.erreurCalculException)
      // Curieusement mathjs n'a pas prévu d'exporter la soustraction
      // res = mathjs.add.subtract(op1, op2)
      res = []
      for (let i = 0; i < n1; i++) {
        const lig = []
        res.push(lig)
        for (let j = 0; j < p1; j++) {
          lig.push(op1.get([i, j]) - op2.get([i, j]))
        }
      }
      res = mathjs.matrix(res)
      size = res.size()
      if (size[0] === 1 && size[1] === 1) return res.get([0, 0])
      return res
    case Ope.Divi: // Division terme à terme
      if (n1 !== n2 || p1 !== p2) throw new Error(kernel.erreurCalculException)
      // Curieusement mathjs n'a pas prévu d'exporter la soustraction
      // res = mathjs.add.subtract(op1, op2)
      res = []
      for (let i = 0; i < n1; i++) {
        const lig = []
        res.push(lig)
        for (let j = 0; j < p1; j++) {
          const n = op1.get([i, j])
          const d = op2.get([i, j])
          if (d === 0) throw new Error(kernel.erreurCalculException)
          lig.push(n / d)
        }
      }
      res = mathjs.matrix(res)
      size = res.size()
      if (size[0] === 1 && size[1] === 1) return res.get([0, 0])
      return res

    default :
      throw new Error(kernel.erreurCalculException)
  }
}

/**
 * Fonction appliquant l'opération de base entre une matrice et un nombre
 * @param {number} nb Le nombre
 * @param {matrix} mat La matrice
 * @param nbleft true si la matrice est l'opérande de gauche, false sinon
 * @return {matrix}
 */
COperation.prototype.applyToMat = function (nb, mat, nbleft) {
  const size = mat.size()
  const n = size[0]
  const p = size[1]
  const res = []
  for (let i = 0; i < n; i++) {
    const lig = []
    for (let j = 0; j < p; j++) {
      lig.push(this.resultatBase(nbleft ? nb : mat.get([i, j]), nbleft ? mat.get([i, j]) : nb))
    }
    res.push(lig)
  }
  return mathjs.matrix(res)
}

COperation.prototype.dependDeVariable = function (ind) {
  return this.operande1.dependDeVariable(ind) || this.operande2.dependDeVariable(ind)
}

COperation.prototype.getCopie = function () {
  return new COperation(this.listeProprietaire, this.operande1.getCopie(), this.operande2.getCopie(),
    this.ope)
}

COperation.prototype.chaineCalcul = function (varFor) {
  var op1, op2
  var ch1 = ''; var ch2 = ''; var ch = ''
  var parouv = '('
  var parfer = ')'
  switch (this.ope) {
    case Ope.Plus:
      if (this.operande1.nature() === CCbGlob.natOperation) {
        op1 = this.operande1.ope
        if ((op1 === Ope.Plus) || (op1 === Ope.Moin) ||
          (op1 === Ope.Mult) || (op1 === Ope.Divi)) { ch1 = this.operande1.chaineCalculSansPar(varFor) } else ch1 = this.operande1.chaineCalcul(varFor)
      } else {
        // else ch1 = operande1.chaineCalcul(varFor);
        // Rajouté version 4.6
        if (this.operande1.nature() === CCbGlob.natMoinsUnaire) ch1 = this.operande1.chaineCalculSansPar(varFor)
        else ch1 = this.operande1.chaineCalcul(varFor)
      }
      if (this.operande2.nature() === CCbGlob.natOperation) {
        op2 = this.operande2.ope
        if ((op2 === Ope.Plus) || (op2 === Ope.Moin) ||
          (op2 === Ope.Mult) || (op2 === Ope.Divi)) { ch2 = this.operande2.chaineCalculSansPar(varFor) } else ch2 = this.operande2.chaineCalcul(varFor)
      } else ch2 = this.operande2.chaineCalcul(varFor)
      ch = parouv + ch1 + '+' + ch2 + parfer
      break
    case Ope.Moin:
      if (this.operande1.nature() === CCbGlob.natOperation) {
        op1 = this.operande1.ope
        if ((op1 === Ope.Plus) || (op1 === Ope.Moin) ||
          (op1 === Ope.Mult) || (op1 === Ope.Divi)) { ch1 = this.operande1.chaineCalculSansPar(varFor) } else ch1 = this.operande1.chaineCalcul(varFor)
      } else {
        // ch1 = operande1.chaineCalcul(varFor);
        // Rajouté version 4.6
        if (this.operande1.nature() === CCbGlob.natMoinsUnaire) ch1 = this.operande1.chaineCalculSansPar(varFor)
        else ch1 = this.operande1.chaineCalcul(varFor)
      }
      if (this.operande2.nature() === CCbGlob.natOperation) {
        op2 = this.operande2.ope
        if ((op2 === Ope.Mult) || (op2 === Ope.Divi)) { ch2 = this.operande2.chaineCalculSansPar(varFor) } else ch2 = this.operande2.chaineCalcul(varFor)
      } else ch2 = this.operande2.chaineCalcul(varFor)
      ch = parouv + ch1 + '-' + ch2 + parfer
      break
    case Ope.Mult:
      if (this.operande1.nature() === CCbGlob.natOperation) {
        op1 = this.operande1.ope
        if ((op1 === Ope.Mult) || (op1 === Ope.Divi)) { ch1 = this.operande1.chaineCalculSansPar(varFor) } else ch1 = this.operande1.chaineCalcul(varFor)
      } else ch1 = this.operande1.chaineCalcul(varFor)
      if (this.operande2.nature() === CCbGlob.natOperation) {
        op2 = this.operande2.ope
        if (op2 === Ope.Mult) { ch2 = this.operande2.chaineCalculSansPar(varFor) } else ch2 = this.operande2.chaineCalcul(varFor)
      } else ch2 = this.operande2.chaineCalcul(varFor)
      ch = parouv + ch1 + '*' + ch2 + parfer
      break
    case Ope.Divi:
      if (this.operande1.nature() === CCbGlob.natOperation) {
        op1 = this.operande1.ope
        if ((op1 === Ope.Mult) || (op1 === Ope.Divi)) { ch1 = this.operande1.chaineCalculSansPar(varFor) } else ch1 = this.operande1.chaineCalcul(varFor)
      } else ch1 = this.operande1.chaineCalcul(varFor)
      ch2 = this.operande2.chaineCalcul(varFor)
      ch = parouv + ch1 + '/' + ch2 + parfer
      break
    case Ope.Inf:
    case Ope.Sup:
    case Ope.InfOuEgal:
    case Ope.SupOuEgal:
    case Ope.Egalite:
    case Ope.Diff:
      if (this.operande1.nature() === CCbGlob.natOperation) {
        // var oper1 = operande1;
        if (Ope.estTest(this.operande1.ope)) ch1 = this.operande1.chaineCalcul(varFor)
        else ch1 = this.operande1.chaineCalculSansPar(varFor)
      } else ch1 = this.operande1.chaineCalculSansPar(varFor)
      if (this.operande2.nature() === CCbGlob.natOperation) {
        // var oper2 = operande2;
        if (Ope.estTest(this.operande2.ope)) ch2 = this.operande2.chaineCalcul(varFor)
        else ch2 = this.operande2.chaineCalculSansPar(varFor)
      } else ch2 = this.operande2.chaineCalculSansPar(varFor)
      ch = parouv + ch1 + Ope.chaineOperateur(this.ope) + ch2 + parfer
      break
    case Ope.And: // Modifié version 6.3.4
      if (this.operande1.nature() === CCbGlob.natOperation) {
        op1 = this.operande1.ope
        if (op1 === Ope.Or) { ch1 = this.operande1.chaineCalcul(varFor) } else ch1 = this.operande1.chaineCalculSansPar(varFor)
      } else ch1 = this.operande1.chaineCalcul(varFor)
      if (this.operande2.nature() === CCbGlob.natOperation) {
        op2 = this.operande2.ope
        if (op2 === Ope.Or) { ch2 = this.operande2.chaineCalcul(varFor) } else ch2 = this.operande2.chaineCalculSansPar(varFor)
      } else ch2 = this.operande2.chaineCalcul(varFor)
      ch = parouv + ch1 + Ope.chaineOperateur(this.ope) + ch2 + parfer
      break
    case Ope.Or:
      ch1 = this.operande1.chaineCalculSansPar(varFor)
      ch2 = this.operande2.chaineCalculSansPar(varFor)
      ch = parouv + ch1 + Ope.chaineOperateur(this.ope) + ch2 + parfer
  } // switch Ope}
  // Au cas où le calcul commençait par un moins unaire, la chaine commence par "0-"
  if (ch.substring(0, 3) === '(0-') ch = parouv + '-' + ch.substring(3)
  return ch
}
COperation.prototype.chaineCalculSansPar = function (varFor) {
  var op1, op2
  var ch1 = ''; var ch2 = ''; var ch = ''
  switch (this.ope) {
    case Ope.Plus:
      if (this.operande1.nature() === CCbGlob.natOperation) {
        op1 = this.operande1.ope
        if ((op1 === Ope.Plus) || (op1 === Ope.Moin) ||
          (op1 === Ope.Mult) || (op1 === Ope.Divi)) { ch1 = this.operande1.chaineCalculSansPar(varFor) } else ch1 = this.operande1.chaineCalcul(varFor)
      } else {
        if (this.operande1.nature() === CCbGlob.natMoinsUnaire) ch1 = this.operande1.chaineCalculSansPar(varFor)
        else ch1 = this.operande1.chaineCalcul(varFor)
      }
      if (this.operande2.nature() === CCbGlob.natOperation) {
        op2 = this.operande2.ope
        if ((op2 === Ope.Plus) || (op2 === Ope.Moin) ||
          (op2 === Ope.Mult) || (op2 === Ope.Divi)) { ch2 = this.operande2.chaineCalculSansPar(varFor) } else ch2 = this.operande2.chaineCalcul(varFor)
      } else ch2 = this.operande2.chaineCalcul(varFor)
      ch = ch1 + '+' + ch2
      break
    case Ope.Moin:
      if (this.operande1.nature() === CCbGlob.natOperation) {
        op1 = this.operande1.ope
        if ((op1 === Ope.Plus) || (op1 === Ope.Moin) ||
          (op1 === Ope.Mult) || (op1 === Ope.Divi)) { ch1 = this.operande1.chaineCalculSansPar(varFor) } else ch1 = this.operande1.chaineCalcul(varFor)
      } else {
        if (this.operande1.nature() === CCbGlob.natMoinsUnaire) ch1 = this.operande1.chaineCalculSansPar(varFor)
        else ch1 = this.operande1.chaineCalcul(varFor)
      }
      if (this.operande2.nature() === CCbGlob.natOperation) {
        op2 = this.operande2.ope
        if ((op2 === Ope.Mult) || (op2 === Ope.Divi)) { ch2 = this.operande2.chaineCalculSansPar(varFor) } else ch2 = this.operande2.chaineCalcul(varFor)
      } else ch2 = this.operande2.chaineCalcul(varFor)
      ch = ch1 + '-' + ch2
      break
    case Ope.Mult:
      if (this.operande1.nature() === CCbGlob.natOperation) {
        op1 = this.operande1.ope
        if ((op1 === Ope.Mult) || (op1 === Ope.Divi)) { ch1 = this.operande1.chaineCalculSansPar(varFor) } else ch1 = this.operande1.chaineCalcul(varFor)
      } else ch1 = this.operande1.chaineCalcul(varFor)
      if (this.operande2.nature() === CCbGlob.natOperation) {
        op2 = this.operande2.ope
        if (op2 === Ope.Mult) { ch2 = this.operande2.chaineCalculSansPar(varFor) } else ch2 = this.operande2.chaineCalcul(varFor)
      } else ch2 = this.operande2.chaineCalcul(varFor)
      ch = ch1 + '*' + ch2
      break
    case Ope.Divi:
      if (this.operande1.nature() === CCbGlob.natOperation) {
        op1 = this.operande1.ope
        if ((op1 === Ope.Mult) || (op1 === Ope.Divi)) { ch1 = this.operande1.chaineCalculSansPar(varFor) } else ch1 = this.operande1.chaineCalcul(varFor)
      } else ch1 = this.operande1.chaineCalcul(varFor)
      ch2 = this.operande2.chaineCalcul(varFor)
      ch = ch1 + '/' + ch2
      break
    // Modifié version 3.0
    case Ope.Inf:
    case Ope.Sup:
    case Ope.InfOuEgal:
    case Ope.SupOuEgal:
    case Ope.Egalite:
    case Ope.Diff:
      if (this.operande1.nature() === CCbGlob.natOperation) {
        // var oper1 = operande1;
        if (Ope.estTest(this.operande1.ope)) ch1 = this.operande1.chaineCalcul(varFor)
        else ch1 = this.operande1.chaineCalculSansPar(varFor)
      } else ch1 = this.operande1.chaineCalculSansPar(varFor)
      if (this.operande2.nature() === CCbGlob.natOperation) {
        // var oper2 = operande2;
        if (Ope.estTest(this.operande2.ope)) ch2 = this.operande2.chaineCalcul(varFor)
        else ch2 = this.operande2.chaineCalculSansPar(varFor)
      } else ch2 = this.operande2.chaineCalculSansPar(varFor)
      ch = ch1 + Ope.chaineOperateur(this.ope) + ch2
      break
    case Ope.And: // Modifié version 6.3.4
      if (this.operande1.nature() === CCbGlob.natOperation) {
        op1 = this.operande1.ope
        if (op1 === Ope.Or) { ch1 = this.operande1.chaineCalcul(varFor) } else ch1 = this.operande1.chaineCalculSansPar(varFor)
      } else ch1 = this.operande1.chaineCalcul(varFor)
      if (this.operande2.nature() === CCbGlob.natOperation) {
        op2 = this.operande2.ope
        if (op2 === Ope.Or) { ch2 = this.operande2.chaineCalcul(varFor) } else ch2 = this.operande2.chaineCalculSansPar(varFor)
      } else ch2 = this.operande2.chaineCalcul(varFor)
      ch = ch1 + Ope.chaineOperateur(this.ope) + ch2
      break
    case Ope.Or:
      ch1 = this.operande1.chaineCalculSansPar(varFor)
      ch2 = this.operande2.chaineCalculSansPar(varFor)
      ch = ch1 + Ope.chaineOperateur(this.ope) + ch2
  } // switch Ope}
  // Pour un calcul complexe nul la chaîne renvoyée est "0+0*i" on renvoie alors "0"
  if (ch === COperation.zeroComp) return '0'
  // Au cas où le calcul commençait par un moins unaire, la chaine commence par "0-"
  if (ch.substring(0, 2) === '0-') ch = ch.substring(1)

  return ch
}

COperation.prototype.chaineLatex = function (varFor) {
  return '\\left(' + this.chaineLatexSansPar(varFor) + '\\right)'
}

COperation.prototype.chaineLatexSansPar = function (varFor) {
  var op1, op2
  var ch1 = ''; var ch2 = ''; var ch = ''
  switch (this.ope) {
    case Ope.Plus:
      if (this.operande1.nature() === CCbGlob.natOperation) {
        op1 = this.operande1.ope
        if ((op1 === Ope.Plus) || (op1 === Ope.Moin) ||
          (op1 === Ope.Mult) || (op1 === Ope.Divi)) { ch1 = this.operande1.chaineLatexSansPar(varFor) } else ch1 = this.operande1.chaineLatex(varFor)
      } else ch1 = this.operande1.chaineLatexSansPar(varFor)
      if (this.operande2.nature() === CCbGlob.natOperation) {
        op2 = this.operande2.ope
        if ((op2 === Ope.Plus) || (op2 === Ope.Moin) ||
          (op2 === Ope.Mult) || (op2 === Ope.Divi)) {
          ch2 = this.operande2.chaineLatexSansPar(varFor)
        } else ch2 = this.operande2.chaineLatex(varFor)
        // } else ch2 = this.operande2.chaineLatex(varFor) // Modifié version 6.4.8
      } else ch2 = this.operande2.chaineLatexSansPar(varFor)
      if (ch2.charAt(0) === '-') ch = ch1 + '-' + ch2.substring(1)
      else ch = ch1 + '+' + ch2
      break
    case Ope.Moin:
      if (this.operande1.nature() === CCbGlob.natOperation) {
        op1 = this.operande1.ope
        if ((op1 === Ope.Plus) || (op1 === Ope.Moin) ||
          (op1 === Ope.Mult) || (op1 === Ope.Divi)) { ch1 = this.operande1.chaineLatexSansPar(varFor) } else ch1 = this.operande1.chaineLatex(varFor)
      } else {
        if (this.operande1.nature() === CCbGlob.natMoinsUnaire) ch1 = this.operande1.chaineLatexSansPar(varFor)
        else ch1 = this.operande1.chaineLatex(varFor)
      }
      if (this.operande2.nature() === CCbGlob.natOperation) {
        op2 = this.operande2.ope
        if ((op2 === Ope.Mult) || (op2 === Ope.Divi)) { ch2 = this.operande2.chaineLatexSansPar(varFor) } else ch2 = this.operande2.chaineLatex(varFor)
      } else ch2 = this.operande2.chaineLatex(varFor)
      ch = ch1 + '-' + ch2
      break
    case Ope.Mult:
      var bsignemult = false
      if (this.operande1.nature() === CCbGlob.natOperation) {
        op1 = this.operande1.ope
        if ((op1 === Ope.Mult) || (op1 === Ope.Divi)) { ch1 = this.operande1.chaineLatexSansPar(varFor) } else ch1 = this.operande1.chaineLatex(varFor)
        // if (op1 == Ope.Divi) bsignemult = true; // Supprimé version 4.9.2
      } else ch1 = this.operande1.chaineLatex(varFor)
      if (this.operande2.nature() === CCbGlob.natOperation) {
        op2 = this.operande2.ope
        if ((op2 === Ope.Mult) || (op2 === Ope.Divi)) { ch2 = this.operande2.chaineLatexSansPar(varFor) } else ch2 = this.operande2.chaineLatex(varFor)
        if (op2 === Ope.Divi) bsignemult = true
      } else ch2 = this.operande2.chaineLatex(varFor)
      // ch = ch1 + "*" + ch2; // Sécial pour Latex, pas de signe multiplicatif
      if (bsignemult || Fonte.chiffre(ch2.charAt(0))) ch = ch1 + '\\times ' + ch2 // Espace de fin nécessaire
      else ch = ch1 + ch2
      break
    case Ope.Divi:
      var bfrac = true
      if (this.operande1.nature() === CCbGlob.natOperation) {
        op1 = this.operande1.ope
        if (op1 === Ope.Mult) {
          bfrac = false
          var operandegauche1 = this.operande1.operande1
          if (operandegauche1.nature() === CCbGlob.natOperation) {
            var opgauche1 = operandegauche1.ope
            if (opgauche1 === Ope.Divi) {
              bfrac = false
              ch1 = this.operande1.operande1.chaineLatexSansPar(varFor)
              ch2 = '\\frac{' + this.operande1.operande2.chaineLatexSansPar(varFor) + '} {' + this.operande2.chaineLatexSansPar(varFor) + '}'
            } else {
              bfrac = true
              ch1 = this.operande1.chaineLatexSansPar(varFor)
              ch2 = this.operande2.chaineLatexSansPar(varFor)
            }
          } else {
            bfrac = true
            ch1 = this.operande1.chaineLatexSansPar(varFor)
            ch2 = this.operande2.chaineLatexSansPar(varFor)
          }
        } else {
          ch1 = this.operande1.chaineLatexSansPar(varFor)
          ch2 = this.operande2.chaineLatexSansPar(varFor)
        }
      } else {
        ch1 = this.operande1.chaineLatexSansPar(varFor)
        ch2 = this.operande2.chaineLatexSansPar(varFor)
      }
      if (bfrac) ch = '\\frac{' + ch1 + '}{' + ch2 + '}'
      else ch = ch1 + '\\times' + ch2

      break
    // Modifié version 3.0
    case Ope.Inf:
    case Ope.Sup:
    case Ope.InfOuEgal:
    case Ope.SupOuEgal:
    case Ope.Egalite:
    case Ope.Diff:
      if (this.operande1.nature() === CCbGlob.natOperation) {
        // var oper1 = operande1;
        if (Ope.estTest(this.operande1.ope)) ch1 = this.operande1.chaineLatex(varFor)
        else ch1 = this.operande1.chaineLatexSansPar(varFor)
      } else {
        // else ch1 = operande1.chaineCalculSansPar(varFor); Correction défaut version 4.7.2
        ch1 = this.operande1.chaineLatexSansPar(varFor)
      }
      if (this.operande2.nature() === CCbGlob.natOperation) {
        // var oper2 = operande2;
        if (Ope.estTest(this.operande2.ope)) ch2 = this.operande2.chaineLatex(varFor)
        else ch2 = this.operande2.chaineLatexSansPar(varFor)
      } else ch2 = this.operande2.chaineLatexSansPar(varFor)

      ch = ch1 + Ope.chaineOperateurLatex(this.ope) + ' ' + ch2
      break
    case Ope.And:
    case Ope.Or:
      ch1 = this.operande1.chaineLatexSansPar(varFor)
      var natop2 = this.operande2.nature()
      if (natop2 === CCbGlob.natOperation) {
        op2 = this.operande2.operateur
        if ((op2 === Ope.And) || (op2 === Ope.Or)) ch2 = this.operande2.chaineLatex(varFor)
        else ch2 = this.operande2.chaineLatexSansPar(varFor)
      } else ch2 = this.operande2.chaineLatexSansPar(varFor)
      ch = ch1 + Ope.chaineOperateurLatex(this.ope) + ch2
  } // switch Ope}
  // Pour un calcul complexe nul la chaîne renvoyée est "0+0*i" on renvoie alors "0"
  if (ch === COperation.zeroComp) return '0'
  // Au cas où le calcul commençait par un moins unaire, la chaine commence par "0-"
  if (ch.substring(0, 2) === '0-') ch = ch.substring(1)

  return ch
}

COperation.prototype.read = function (inps, list) {
  CCb.prototype.read.call(this, inps, list)
  this.ope = inps.readByte()
  this.operande1 = inps.readObject(list)
  this.operande2 = inps.readObject(list)
}

COperation.prototype.write = function (oups, list) {
  CCb.prototype.write.call(this, oups, list)
  oups.writeByte(this.ope)
  oups.writeObject(this.operande1)
  oups.writeObject(this.operande2)
}

COperation.prototype.estConstant = function () {
  return this.operande1.estConstant() && this.operande2.estConstant()
}

COperation.prototype.deriveePossible = function (indiceVariable) {
  return this.operande1.deriveePossible(indiceVariable) && this.operande2.deriveePossible(indiceVariable)
}

COperation.prototype.calculAvecValeursRemplacees = function (bfrac) {
  return new COperation(this.listeProprietaire, this.operande1.calculAvecValeursRemplacees(bfrac),
    this.operande2.calculAvecValeursRemplacees(bfrac), this.ope)
}

COperation.prototype.membreGauche = function () {
  return this.operande1
}

COperation.prototype.membreDroit = function () {
  return this.operande2
}

COperation.prototype.isCalcVect = function (tabNames) {
  const op1 = this.operande1
  const op2 = this.operande2
  switch (this.ope) {
    case Ope.Plus:
    case Ope.Moin:
    case Ope.Egalite: // Une  égamité est considérée comme résultat vectoriel si les deux membres le sont
      return (op1.isCalcVect(tabNames) || op1.isVectNul()) && (op2.isCalcVect(tabNames) || op2.isVectNul())
    case Ope.Mult:
      return !op1.isCalcVect(tabNames) && (op2.isCalcVect(tabNames) || op2.isVectNul()) // On autorise k*vecteur mais pas vecteur*
    default:
      return false
  }
}

COperation.prototype.isCalcOK4Vect = function (tabNames) {
  if (!(this.operande1.isCalcOK4Vect(tabNames) && this.operande2.isCalcOK4Vect(tabNames))) return false
  const op1 = this.operande1
  const op2 = this.operande2
  switch (this.ope) {
    case Ope.Plus:
    case Ope.Moin:
      return (op1.isCalcVect(tabNames) && op2.isCalcVect(tabNames)) || (!op1.isCalcVect(tabNames) && !op2.isCalcVect(tabNames))
    case Ope.Mult:
      // On autorise k*vecteur mais pas vecteur*k
      return (!op1.isCalcVect(tabNames) && op2.isCalcVect(tabNames)) || (!op1.isCalcVect(tabNames) && !op2.isCalcVect(tabNames))
    default:
      return true
  }
}

// Déplacé ici version 6.4.7
COperation.prototype.calculNormalise = function (rempval, sousdivnorm, rempDecParFrac) {
  // Attention : A cause d'un problème webPack les require ci-dessous ne doivent pas être en tête de fichier
  const CConstante = require('./CConstante')
  const CMoinsUnaire = require('./CMoinsUnaire')
  const liste = this.listeProprietaire
  const calcOperande1 = this.operande1.calculNormalise(rempval, sousdivnorm, rempDecParFrac)
  const calcOperande2 = this.operande2.calculNormalise(rempval, sousdivnorm, rempDecParFrac)
  const natop1 = calcOperande1.nature()
  const natop2 = calcOperande2.nature()
  if (this.ope === Ope.Moin) {
    // On remplace les a - 0 par a
    if (rempval) {
      if ((natop2 === CCbGlob.natConstante)) {
        if (calcOperande2.valeur === 0) return calcOperande1
      }
      // On remplace les 0 - a par - a ou par abs(a) si a est un moins unaire
      if ((natop1 === CCbGlob.natConstante)) {
        // CConstante resval = (CConstante) calcOperande1;
        if (calcOperande1.valeur === 0) {
          if (calcOperande2.nature() === CCbGlob.natMoinsUnaire) return calcOperande2.operande
          return new CMoinsUnaire(liste, calcOperande2)
        }
      }
      // On remplace a - (-b) par a + b
      if (natop2 === CCbGlob.natMoinsUnaire) {
        // CMoinsUnaire mun = (CMoinsUnaire) calcOperande2;
        // return new COperation(liste, calcOperande1, mun.operande, Ope.Plus); // Modifié version 4.7.2.5
        return new COperation(liste, calcOperande1, calcOperande2.operande,
          Ope.Plus).calculNormalise(rempval, sousdivnorm, rempDecParFrac)
      }
    }
    // On remplace les a - b par a + (-b)
    if (sousdivnorm) {
      return new COperation(liste, calcOperande1,
        new CMoinsUnaire(liste, calcOperande2), Ope.Plus)
    } else return new COperation(liste, calcOperande1, calcOperande2, Ope.Moin)
  }
  if (this.ope === Ope.Divi) {
    // Si on est en mode remplacement de valeurs et que le dénominateur est un résultat de valeur 1 on simplifie
    if (rempval) {
      if (natop2 === CCbGlob.natConstante) {
        if (calcOperande2.valeur === 1) {
          // Pour 1/1 on renvoie 1
          if (natop1 === CCbGlob.natConstante) {
            if (calcOperande1.valeur === 1) return new CConstante(liste, 1)
          }
          return calcOperande1
        }
      }
      // Si le numérateur est une constante qui vaut 1 on garde le quotient
      if (natop1 === CCbGlob.natConstante) {
        // CConstante resval = (CConstante) calcOperande1;
        var r = calcOperande1.valeur
        if (r === 1) {
          if (calcOperande2.nature() === CCbGlob.natMoinsUnaire) {
            return new CMoinsUnaire(liste, new COperation(liste,
              new CConstante(liste, 1), calcOperande2.operande, Ope.Divi)
              .calculNormalise(rempval, sousdivnorm, rempDecParFrac))
          } else return new COperation(liste, new CConstante(liste, 1), calcOperande2, Ope.Divi)
        }
        // S'il vaut 0 on renvoie un CResultatValeur contenant 0
        if (r === 0) return new CConstante(liste, 0)
      }
      // Si le numérateur ou le dénominateur sont des moins unaires on simplifie
      if (natop1 === CCbGlob.natMoinsUnaire) {
        // CMoinsUnaire mun1 = (CMoinsUnaire) calcOperande1;
        if (natop2 === CCbGlob.natMoinsUnaire) {
          // CMoinsUnaire mun2 = (CMoinsUnaire) calcOperande2;
          if (sousdivnorm) {
            return new COperation(liste, calcOperande1.operande,
              new COperation(liste, new CConstante(liste, 1),
                calcOperande2.operande, Ope.Divi), Ope.Mult).calculNormalise(rempval, sousdivnorm, rempDecParFrac)
          } else {
            return new COperation(liste, calcOperande1.operande, calcOperande2.operande,
              Ope.Divi).calculNormalise(rempval, sousdivnorm, rempDecParFrac)
          }
        }
        if (sousdivnorm) {
          return new CMoinsUnaire(liste, new COperation(liste, calcOperande1.operande,
            new COperation(liste, new CConstante(liste, 1),
              calcOperande2, Ope.Divi), Ope.Mult)).calculNormalise(rempval, sousdivnorm, rempDecParFrac)
        } else {
          return new CMoinsUnaire(liste, new COperation(liste, calcOperande1.operande, calcOperande2,
            Ope.Divi).calculNormalise(rempval, sousdivnorm, rempDecParFrac))
        }
      }
      if (natop2 === CCbGlob.natMoinsUnaire) {
        // CMoinsUnaire mun2 = (CMoinsUnaire) calcOperande2;
        if (sousdivnorm) {
          return new CMoinsUnaire(liste, new COperation(liste, calcOperande1,
            new COperation(liste, new CConstante(liste, 1),
              calcOperande2.operande, Ope.Divi), Ope.Mult)).calculNormalise(rempval, sousdivnorm, rempDecParFrac)
        } else {
          return new CMoinsUnaire(liste, new COperation(liste, calcOperande1,
            calcOperande2.operande, Ope.Divi).calculNormalise(rempval, sousdivnorm, rempDecParFrac))
        }
      }
    }
    // On remplace (-a)/b par - (a/b)
    // Si le numérateur ou le dénominateur sont des moins unaires on simplifie
    if (natop1 === CCbGlob.natMoinsUnaire) {
      // CMoinsUnaire mun1 = (CMoinsUnaire) calcOperande1;
      var natnum = calcOperande1.operande.nature()
      // Si le numérateur est une constante -1 on renvoie -(1/denominateur)
      if (natnum === CCbGlob.natConstante) {
        var c = calcOperande1.operande
        if (c.valeur === 1) {
          return new CMoinsUnaire(liste, new COperation(liste, new CConstante(liste, 1),
            calcOperande2, Ope.Divi))
        }
      }
      if (sousdivnorm) {
        return new CMoinsUnaire(liste, new COperation(liste, calcOperande1.operande,
          new COperation(liste, new CConstante(liste, 1),
            calcOperande2, Ope.Divi), Ope.Mult)).calculNormalise(rempval, sousdivnorm, rempDecParFrac)
      } else {
        return new CMoinsUnaire(liste,
          new COperation(liste, calcOperande1.operande, calcOperande2, Ope.Divi)).calculNormalise(rempval, sousdivnorm, rempDecParFrac)
      }
    }
    // Fin ajout version 4.7.2.5
    // On remplace a / b par a * (1/b) sauf si a est une constante à 1
    if (natop1 === CCbGlob.natConstante) {
      // CConstante cons = (CConstante) calcOperande1;
      if (calcOperande1.valeur === 1) return new COperation(liste, calcOperande1, calcOperande2, Ope.Divi)
    }
    // Si l'opérande de gauche est un produit dont un des facteurs est une constante 1
    // On le remplace par le facteur de gauche
    // La ligne suivante a été modifié version 6.4.7 en rajoutant le test rempval
    // car sinon, en mode non remplacement, (1*a²)/2 et a²/2 étaient considérés comme équivalents
    // Revu version 6.4.7 car avec cette modif, en mode non remplacement, par exemple 1/3*1/3*1/e^3
    // ne sera pas considéré comme équivalent après remplacement à 1/a*1/a*1/e^a avec a = 3.
    // Pour cette raison, si rempval est false, on remplacera pas seulement les produits avec un facteur 1 à
    // gauche
    /*
    if (rempval && (natop1 === CCbGlob.natOperation)) {
      // COperation op1 = (COperation) calcOperande1;
      if (calcOperande1.ope === Ope.Mult) {
        if (calcOperande1.operande1.nature() === CCbGlob.natConstante) {
          if (calcOperande1.operande1.valeur === 1) {
            return new COperation(liste, calcOperande1.operande2, calcOperande2,
              Ope.Divi).calculNormalise(rempval, sousdivnorm, rempDecParFrac, rempDecParFrac)
          }
        }
        if (calcOperande1.operande2.nature() === CCbGlob.natConstante) {
          if (calcOperande1.operande2.valeur === 1) {
            return new COperation(liste, calcOperande1.operande1, calcOperande2,
              Ope.Divi).calculNormalise(rempval, sousdivnorm, rempDecParFrac, rempDecParFrac)
          }
        }
      }
    }
     */
    if (natop1 === CCbGlob.natOperation) {
      if (calcOperande1.ope === Ope.Mult) {
        if (calcOperande1.operande1.nature() === CCbGlob.natConstante) {
          if (calcOperande1.operande1.valeur === 1) {
            return new COperation(liste, calcOperande1.operande2, calcOperande2,
              Ope.Divi).calculNormalise(rempval, sousdivnorm, rempDecParFrac, rempDecParFrac)
          }
        }
        if (calcOperande1.operande2.nature() === CCbGlob.natConstante) {
          if (calcOperande1.operande2.valeur === 1) {
            return new COperation(liste, calcOperande1.operande1, calcOperande2,
              Ope.Divi).calculNormalise(rempval, sousdivnorm, rempDecParFrac, rempDecParFrac)
          }
        }
      }
    }

    if (sousdivnorm) {
      return new COperation(liste, calcOperande1, new COperation(liste,
        new CConstante(liste, 1), calcOperande2, Ope.Divi), Ope.Mult)
    } else return new COperation(liste, calcOperande1, calcOperande2, Ope.Divi)
  }
  if (this.ope === Ope.Plus) {
    if (rempval) {
      if (natop1 === CCbGlob.natConstante) {
        if (calcOperande1.valeur === 0) return calcOperande2
      }
      if (natop2 === CCbGlob.natConstante) {
        if (calcOperande2.valeur === 0) return calcOperande1
      }
    }
    if (sousdivnorm) return new COperation(liste, calcOperande1, calcOperande2, Ope.Plus)
    else {
      // On remplace a + (-b) par a - b
      if (natop2 === CCbGlob.natMoinsUnaire) {
        // CMoinsUnaire mun2 = (CMoinsUnaire) calcOperande2;
        return new COperation(liste, calcOperande1, calcOperande2.operande, Ope.Moin).calculNormalise(rempval, sousdivnorm, rempDecParFrac)
      }
    }
  }
  if (this.ope === Ope.Mult) {
    if (rempval) {
      // Il  faut faire "remonter" les moins unaires le plus haut possible
      if (natop1 === CCbGlob.natMoinsUnaire) {
        if (natop2 === CCbGlob.natMoinsUnaire) {
          return new COperation(liste, calcOperande1.operande, calcOperande2.operande,
            Ope.Mult).calculNormalise(rempval, sousdivnorm, rempDecParFrac)
        } else {
          return new CMoinsUnaire(liste,
            new COperation(liste, calcOperande1.operande, calcOperande2,
              Ope.Mult).calculNormalise(rempval, sousdivnorm, rempDecParFrac)).calculNormalise(rempval, sousdivnorm, rempDecParFrac)
        } // Modifié version 6.3.4
      } else {
        if (natop2 === CCbGlob.natMoinsUnaire) {
          return new CMoinsUnaire(liste, new COperation(liste, calcOperande1,
            calcOperande2.operande, Ope.Mult).calculNormalise(rempval, sousdivnorm, rempDecParFrac)).calculNormalise(rempval, sousdivnorm, rempDecParFrac) // Modifié version 6.3.4;
        }
      }
      if (natop1 === CCbGlob.natConstante) {
        r = calcOperande1.valeur
        if (r === 1) return calcOperande2
        if (r === 0) return new CConstante(liste, 0)
      }
      if (natop2 === CCbGlob.natConstante) {
        // CConstante resval2 = (CConstante) calcOperande2;
        r = calcOperande2.valeur
        if (r === 1) return calcOperande1
        if (r === 0) return new CConstante(liste, 0)
      }
      return new COperation(liste, calcOperande1, calcOperande2, Ope.Mult)
    } else {
      // Lignes suivantes ajoutées version 6.3.4 pour accepter par exemple ((-3)/5)*x au lieu de -((3/5)*x)
      if (natop1 === CCbGlob.natMoinsUnaire) {
        var oper1 = calcOperande1.operande
        var natoper1 = oper1.nature()
        if (natoper1 === CCbGlob.natOperation) {
          if (oper1.ope === Ope.Mult) {
            var calc1 = oper1.operande1
            if (calc1.nature() === CCbGlob.natConstante) {
              var calc2 = oper1.operande2
              if (calc2.nature() === CCbGlob.natOperation && calc2.ope === Ope.Divi) {
                if (calc2.operande1.nature() === CCbGlob.natConstante &&
                  calc2.operande1.valeur === 1 && calc2.operande2.nature() === CCbGlob.natConstante) { return new CMoinsUnaire(liste, new COperation(liste, oper1, calcOperande2, Ope.Mult)) }
              }
            }
          } else { // Cas des -1/constante
            if (oper1.ope === Ope.Divi) {
              if (oper1.operande1.nature() === CCbGlob.natConstante && oper1.operande1.valeur === 1 &&
                oper1.operande2.nature() === CCbGlob.natConstante) { return new CMoinsUnaire(liste, new COperation(liste, oper1, calcOperande2, Ope.Mult)) }
            }
          }
        }
      }
    }
  }
  // Autres cas . Rien  à simplifier et on simplifie seulement les opérandes.
  return new COperation(liste, calcOperande1, calcOperande2, this.ope)
}
