/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const NatObj = require('../types/NatObj')
const Pointeur = require('../types/Pointeur')
const kernel = require('../kernel/kernel')
const CPointAncetrePointsMobiles = require('./CPointAncetrePointsMobiles')
module.exports = CPointInterieurPolygone

/**
 * Point intérieur à un polygone
 * @constructor
 * @extends CPointAncetrePointsMobiles
 * @param {CListeObjets} listeProprietaire  La liste propriétaire de l'objet
 * @param {CImplementationProto} impProto  null ou pointe sur la construction propriétaire de l'objet.
 * @param {boolean} estElementFinal  true si l'objet est un objet final de construction
 * @param {Color} couleur  La couleur de l'objet
 * @param {boolean} nomMasque  true si le nom de l'objet est masqué
 * @param {number} decX  Decalage horizontal du nom
 * @param {number} decY  Decalage vertical du nom
 * @param {boolean} masque  true si l'objet est masque
 * @param {String} nom  Le nom eventuel de l'objet
 * @param {number} tailleNom  Indice de la taille du nom (voir Fonte)
 * @param {MotifPoint} motif  Le motif du point
 * @param {boolean} marquePourTrace  true si le point est marqué pour la trace
 * @param {boolean} pointLibre  true si le point n'est pas punaisé
 * @param {CPolygone} polygoneAssocie  Le polygone auquel le point est lié
 * @param {boolean} estSurCote  true si le point est sur un côté
 * @param {number} indicePremierSommet  L'indice du premier segment sur lequel est le point ou alors l'indice
 * du premier des trois sommets du type BCD tels que le point soit intérieur à BCD et CDE
 * @param {Number} alpha  Le coefficient barycentrique normalisé du premier point
 * @param {Numer} beta  Le coefficient barycentrique normalisé du second point
 * @param {Numer} gamma  Le coefficient barycentrique normalisé du troisième point
 * @returns {undefined}
 */
function CPointInterieurPolygone (listeProprietaire, impProto, estElementFinal, couleur, nomMasque,
  decX, decY, masque, nom, tailleNom, motif, marquePourTrace, pointLibre,
  polygoneAssocie, estSurCote, indicePremierSommet, alpha, beta, gamma) {
  if (arguments.length === 1) CPointAncetrePointsMobiles.call(this, listeProprietaire)
  else {
    CPointAncetrePointsMobiles.call(this, listeProprietaire, impProto, estElementFinal, couleur, nomMasque,
      decX, decY, masque, nom, tailleNom, motif, marquePourTrace, pointLibre)
    this.polygoneAssocie = polygoneAssocie
    this.estSurCote = estSurCote
    this.indicePremierSommet = indicePremierSommet
    this.alpha = alpha
    this.beta = beta
    this.gamma = gamma
  }
}
CPointInterieurPolygone.prototype = new CPointAncetrePointsMobiles()
CPointInterieurPolygone.prototype.constructor = CPointInterieurPolygone
CPointInterieurPolygone.prototype.superClass = 'CPointAncetrePointsMobiles'
CPointInterieurPolygone.prototype.className = 'CPointInterieurPolygone'

CPointInterieurPolygone.prototype.getClone = function (listeSource, listeCible) {
  var ind1 = listeSource.indexOf(this.polygoneAssocie)
  var ind2 = listeSource.indexOf(this.impProto)
  return new CPointInterieurPolygone(listeCible, listeCible.get(ind2, 'CImplementationProto'),
    this.estElementFinal, this.couleur, this.nomMasque, this.decX, this.decY, this.masque, this.nom,
    this.tailleNom, this.motif, this.marquePourTrace, this.pointLibre, listeCible.get(ind1, 'CPolygone'),
    this.estSurCote, this.indicePremierSommet, this.alpha, this.beta, this.gamma)
}

CPointInterieurPolygone.prototype.getNature = function () {
  return NatObj.NPointInterieurPolygone
}

CPointInterieurPolygone.prototype.ajouteAntecedents = function (liste) {
  liste.add(this.polygoneAssocie)
}

CPointInterieurPolygone.prototype.depDe = function (p) {
  if (this.elementTestePourDependDe === p) return this.dependDeElementTeste
  return this.memDep(CPointAncetrePointsMobiles.prototype.depDe.call(this, p) || this.polygoneAssocie.depDe(p))
}

CPointInterieurPolygone.prototype.dependDePourBoucle = function (p) {
  return (p === this) || this.polygoneAssocie.dependDePourBoucle(p)
}
CPointInterieurPolygone.prototype.positionne = function (infoRandom, dimfen) {
  var xn, yn
  this.existe = this.polygoneAssocie.existe
  if (!this.existe) return
  var n = this.polygoneAssocie.colPoints[0]
  var a = n.pointeurSurPoint
  n = this.polygoneAssocie.colPoints[this.indicePremierSommet]
  var b = n.pointeurSurPoint
  n = this.polygoneAssocie.colPoints[this.indicePremierSommet + 1]
  var c = n.pointeurSurPoint
  if (this.estSurCote) {
    xn = b.x * this.alpha + c.x * this.beta
    yn = b.y * this.alpha + c.y * this.beta
    CPointAncetrePointsMobiles.prototype.placeEn.call(this, xn, yn)
  } else {
    xn = a.x * this.alpha + b.x * this.beta + c.x * this.gamma
    yn = a.y * this.alpha + b.y * this.beta + c.y * this.gamma
    if (!this.polygoneAssocie.pointInterieur(xn, yn)) {
      var d = this.beta + this.gamma
      xn = (b.x * this.beta + c.x * this.gamma) / d
      yn = (b.y * this.beta + c.y * this.gamma) / d
    }
    CPointAncetrePointsMobiles.prototype.placeEn.call(this, xn, yn)
  }
  CPointAncetrePointsMobiles.prototype.positionne.call(this, infoRandom, dimfen) // Ajouté version 4.6.4
}

CPointInterieurPolygone.prototype.placeEn = function (xn, yn) {
  var estSurCote1 = new Pointeur(false)
  var indicePremierSommet1 = new Pointeur(0)
  var alpha1 = new Pointeur(0)
  var beta1 = new Pointeur(0)
  var gamma1 = new Pointeur(0)
  this.polygoneAssocie.determinePositionInterieure(xn, yn, indicePremierSommet1, estSurCote1, alpha1, beta1, gamma1)
  this.estSurCote = estSurCote1.getValue()
  this.indicePremierSommet = indicePremierSommet1.getValue()
  this.alpha = alpha1.getValue()
  this.beta = beta1.getValue()
  this.gamma = gamma1.getValue()
  CPointAncetrePointsMobiles.prototype.placeEn.call(this, xn, yn)
}

CPointInterieurPolygone.prototype.testDeplacement = function (dimfen, xtest, ytest, pointr, abscr) {
  // On regarde si le point testé est bien intérieur au polygone
  if (this.polygoneAssocie.pointInterieur(xtest, ytest)) {
    pointr.x = xtest
    pointr.y = ytest
    return true
  } else return false
}

CPointInterieurPolygone.prototype.confonduAvec = function (p) {
  if (p.className === this.className) {
    if (kernel.zero(p.x - this.x) && kernel.zero(p.y - this.y)) return (this.polygoneAssocie === p.polygoneAssocie)
    else return false
  } else return false
}

CPointInterieurPolygone.prototype.read = function (inps, list) {
  CPointAncetrePointsMobiles.prototype.read.call(this, inps, list)
  var ind1 = inps.readInt()
  this.polygoneAssocie = list.get(ind1, 'CDroiteAncetre')
  this.estSurCote = inps.readBoolean()
  this.indicePremierSommet = inps.readInt()
  this.alpha = inps.readDouble()
  this.beta = inps.readDouble()
  this.gamma = inps.readDouble()
}

CPointInterieurPolygone.prototype.write = function (oups, list) {
  CPointAncetrePointsMobiles.prototype.write.call(this, oups, list)
  var ind1 = list.indexOf(this.polygoneAssocie)
  oups.writeInt(ind1)
  oups.writeBoolean(this.estSurCote)
  oups.writeInt(this.indicePremierSommet)
  oups.writeDouble(this.alpha)
  oups.writeDouble(this.beta)
  oups.writeDouble(this.gamma)
}
